 /***********************************************************************
 * motor.h
 *
 *  Created on: 24/11/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	motor.h
 * Description:
 * 			This library provides with funtions to manage
 *			the dual motor driver Qik2s9v1 and some other
 * 			funcionalities of Vilma's prototype
 *
 ***********************************************************************/
 
 
#ifndef MOTOR_H_
#define MOTOR_H_



#define MAXMOVEMENTS	10
/*Definition of pin that will be used to perform a driver reset*/
#define MOTOR_RESET_PIN						( 21 )
#define MOTOR_BAUDRATE						(38400)

/*Definition of Motor output that will be connected to the transmission control*/
#define DIRMOTOR 							( 0 )
/* Definition of motor out that will be connected to the direccion control*/
#define TURNMOTOR 							( 1 )

/*Control commands to the qik2s91 Motor driver control */
#define MOTOR_INIT_CMD			         0xAA //170
#define DIRMOTOR_STOP_CMD		         0x86 //134
#define TURNMOTOR_STRAIGHTEN_CMD         0x87 //135
#define DIRMOTOR_FORWARD_CMD             0x88 //136
#define DIRMOTOR_BACKWARD_CMD            0x8A //138
#define TURNMOTOR_RIGHT_CMD           	 0x8C //140
#define TURNMOTOR_LEFT_CMD	             0x8E //142



/*
 * The structure which defines that a movement command are made up by
 * three members such:
 * motor: Could be motor 1 which will control the movement from forward/backward
 * 		and the motor 2 that manages the left and right movement
 *
 * direction: it only could be 'f' to go forward or left and 'b' to go backwards
 * 			 or right
 * speed: It could be from 0 (stopped) to 9 (full speed). In the case of
 * 		  turns the speed will only be fixed as 9 (for motor 2)
 *
 * */
typedef struct {
	uint16_t move;
	uint16_t turn;
	uint8_t speed;
	uint8_t time;

}movements;

/*****************************************************************************
** Function name:		motorInit
**
** Descriptions:		Initializes the uart port and the baudRate
** 						to talk to the motor driver and sends the command a
** 						MOTOR_INIT_CMD to ends up with the baudrate autodetection
** 						doing by default by the driver
**
**
** @parameters:			uart: UART assigned to communicate to the device
**
** @Return:		1 if init was successful
** 				0 Otherwise
**
*****************************************************************************/
uint8_t motorInit(uint32_t uart);

/*****************************************************************************
** Function name:		motorStop
**
** Descriptions:	Set the motor 0 (DIRMOTOR) output to coast (High Z)
** 					stopping the motor
**
** @parameters:			void
** @Returned value:		void
**
*****************************************************************************/
void motorStop();

/*****************************************************************************
** Function name:		motorForward
**
** @Descriptions:		Set the DIRMOTOR to spin Forward with the given speed
** 						and for the time passed as a parameter.
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorForward(uint8_t speed,uint8_t time);

/*****************************************************************************
** Function name:		motorBackward
**
** @Descriptions:		Set the DIRMOTOR (M0) to spin Backwards with the given speed
** 						and for the time passed as a parameter.
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorBackward(uint8_t speed,uint8_t time);

/*****************************************************************************
** Function name:		motorForwardLeft
**
** @Descriptions:		Set the TURNMOTOR (M1) to turns to the left and the DIRMOTOR
** 					    to go forward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorForwardLeft(uint8_t speed,uint8_t time);

/*****************************************************************************
** Function name:		motorForwardRight
**
** @Descriptions:		Set the TURNMOTOR to turns to the right and the DIRMOTOR
** 					    to go forward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorForwardRight(uint8_t speed,uint8_t time);

/*****************************************************************************
** Function name:		motorBackwardRight
**
** @Descriptions:		Set the TURNMOTOR to turns to the right and the DIRMOTOR
** 					    to go backward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorBackwardRight(uint8_t speed,uint8_t time);

/*****************************************************************************
** Function name:		motorBackwardLeft
**
** @Descriptions:		Set the TURNMOTOR to turns to the left and the DIRMOTOR
** 					    to go backward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorBackwardLeft(uint8_t speed,uint8_t time);

/*****************************************************************************
 * Function name: motorUndoMovement
 *
 * Descriptions: This function performs a rollback  of last movement executed
 *
 *
 * parameters:			void
 *  Returned value:		void
 *****************************************************************************/
void motorUndoMovement();

/*****************************************************************************
 * Function name: motoTests
 *
 * Descriptions: This function executes motor turn tests on both motors to make sure
 * 				 they are working and properly connected
 *
 * 	*
 *
 * parameters:			void
 *  Returned value:		void
 *****************************************************************************/
void motorTests();





#endif /* MOTOR_H_ */
