 /***********************************************************************
 * wifly.h
 *
 *  Created on: 22/10/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	wifly.h
 * Description:
 * 			This library contains functions to manage the 
 *			device RN171XV (Wifly) allowing its configuration as well as
 *			get important information about its state
 *
 ***********************************************************************/

#ifndef WIFLY_H_
#define WIFLY_H_



/********************************************************************/

/*                    COMMAND MESSAGES                              */
#define WF_CMD				"$$$"
#define WF_EXIT				"exit"
#define WF_OK				"AOK"
#define WF_ERROR			"ERR"
#define WF_DISCONNECT_WLAN	"leave"
#define WF_SAVE_CONFIG		"save"
#define WF_SOCKET_OPEN		"open"
#define WF_SOCKET_CLOSE		"close"
/********************************************************************/

/*                    ERROR MESSAGES                               */

#define WF_CMD_ON			"Device is already in command mode!"
#define WF_CMD_OFF			"Device is not in command mode! "
#define WF_NOT_CONECTED_MSG	"Device not connected to any WLAN !"
#define WF_SERVER_ERROR		"The server is not available"
#define WF_GENERIC_ERROR	"Something went wrong, please double check"
#define WF_HTTP_ERROR		"The HTTP server connection has failed, please check"
#define WF_HTTP_SETUP_ERROR "The device has not been set up properly"

/********************************************************************/


/*****************************************************************************
 * Function name: wiflyInit
 *
 * Descriptions:Initialize the device by setting up the UART port that is going
 * 				to be used or managing the device, also performs a hardware
 * 				reset to the device
 *
 * parameters:
 * 			uart:uart port to be used
 * 			br:baudrate
 *
 * Returned value:
 * 				1 if Initialization is correct
 *              0 Otherwise
 *****************************************************************************/
uint8_t wiflyInit(uint8_t uart,uint32_t br);


/*****************************************************************************
 ** Function name:		wiflyConnect
 **
 ** Descriptions:		Makes a connection to a wireless network
 **
 ** parameters:			ssid: ID of the wifi lan to connect
 ** 						passwd: security password of the wireless lan
 ** 						enumeration type of authentication
 **
 ** Returned value:		1 if succeeded
 **					    0 Otherwise
 **
 ****************************************************************************/
uint8_t wiflyConnect(char *ssid,char *passwd,uint8_t mode);

/*****************************************************************************
 ** Function name:		wiflySetBaudrate
 **
 ** Descriptions:		Function to change the Baudrate of the device
 **
 **
 ** parameters:			uint 8: New BaudRate
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetBaudRate(uint32_t br);

/*****************************************************************************
 ** Function name:		wiflySetTxPw
 **
 ** Descriptions:		Function to change the Transmission power of the
 ** 						Wifly
 **
 **
 ** parameters:			uint 8: New transmission power
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetTxPw(uint8_t newPw);

/*****************************************************************************
 ** Function name:		wiflySetWlanRate
 **
 ** Descriptions:		Function that changes the work rate for connection
 ** 					between the wifly and the wireless access point
 ** 					fulfilling the formula: rate of a v given value is
 ** 					v= Mbps[v-1]+12Mbps
 **
 ** 					Possible values are:
 **
 ** 					3------11Mbps
 ** 					12-----24Mbps (default)
 ** 					15-----54Mbps
 **
 **
 ** parameters:			uint 8: NewRate
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetWlanRate(uint8_t newRate);

/*****************************************************************************
 ** Function name:		WiflySetHotSpot
 **
 ** Descriptions: This function enables the built-in functionality of Software
 ** 				 Access Point mode. In this mode the wifly becomes a DHCP server
 ** 				 an runs a web application that allows you to configure in a
 ** 				 convenient way several parameters such as the SSID and password
 ** 				 in order the wifly can connect to different AP.
 **
 ** 				 The AP' SSID to associated to is WiFly-EZX-02,once you have
 ** 				 associated to it,the DHPC server will leases a 192.168.1.X address.
 **
 ** 				 Then the configuration page should be accessible through a Internet
 ** 				 browser pointing at http://192.168.1.1
 **
 **
 ** parameters:	 void
 **
 **
 ** Returned value:	void
 **
 **
 ****************************************************************************/
void WiflySetHotSpot();

/*****************************************************************************
 ** Function name:		wiflySocketSetup
 **
 ** Descriptions: Function that sets up the common configuration to establish
 ** 				 a connection to a remote generic server
 **
  parameters:			uint8_t protocol: type of socket.
 ** 									HTTP,TCP client/server UDP etc.
 ** 					uint8_t remoteIp: The remote host ip address
 ** 						 				(0	if dns name is active)
 **
 ** 					uint8_t portRemote: remote socket port
 **
 ** 					char* host: In case of connection must be done
 ** 								by the server FQDN address
 **
 ** Returned value:		uint8_t  1: If the connection was successful
 ** 							 0: Otherwise
 **
 **
 ****************************************************************************/
uint8_t wiflySocketSetup(uint8_t protocol,uint16_t remoteIp,uint16_t portRemote,uint16_t portLocal,char* host);

/*****************************************************************************
 ** Function name:		wiflySendCommand
 **
 ** Descriptions:		Sends a given command to the wifly devices and capture
 ** 						its response.Also,the function permits to enable the debugger
 ** 						 mode for the command sent.
 **  **
 ** 						Example:
 ** 						Sending the command set ip prococol 2 to wifly should
 ** 						be replied as AOK token, We would use:
 **
 ** 						printfSendCommand ("set ip protocol 2",AOK,1)
 **
 **
 ** parameters:
 ** 					char *  commandToSend: the command to send via UART
 ** 					char *  expectedReply: The token expected to receive
 ** 					uint8_t debugMode:1 to send debug messages
 ** 									  0 if don't
 *
 ** Returned value:		uint8_t 1 if command worked fine
 ** 								0 if there were problems
 **
 ****************************************************************************/
uint32_t wiflySendCommand( char* commandToSend,char* expectedReply,uint8_t debugMode);

/*****************************************************************************
 ** Function name:		wiflySendPing
 **
 ** Descriptions:		Function that sends a ping to the host passed as
 ** 						parameter and prints out the result of the ping comand
 ** 						 through the UART port
 **
 ** parameters:			char*: Host to send the ICMP packets
 ** 					uint_8t:number of pings to be sent
 *
 ** Returned value:		uint32_t
 **
 ****************************************************************************/
uint32_t wiflySendPing(char* host,uint8_t times);

/*****************************************************************************
 ** Function name:		wiflyGetIp
 **
 ** Descriptions:		Function that shows the devices' ip address through
 ** 						the UART port
 **
 ** parameters:			char ip: Variable to store the ip address
 *
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflyGetIp(char* ip);

/*****************************************************************************
 ** Function name:		wiflyGetSSID
 **
 ** Descriptions:		Function that gets the ssid where the wifly is
 ** 					associated
 **
 ** parameters:			char ssid: Variable to store the ssid names
 *
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflyGetSSID(char* ssid);

/*****************************************************************************
 ** Function name:		wiflyGetPort
 **
 ** Descriptions:		Function that gives the devices' port
 **
 ** parameters:			void
 *
 ** Returned value:		uint16_t: wifly's port
 **
 ****************************************************************************/
uint16_t wiflyGetPort();

/*****************************************************************************
 ** Function name:		wiflyGetTxPw
 **
 ** Descriptions:		Prints current transmission power through
 ** 						the UART port
 **
 **
 ** parameters:			void
 *
 ** Returned value:		void
 **
 ***************************************************************************/
uint32_t wiflyGetTxPw();

/*****************************************************************************
 ** Function name:		wiflyGetWlanRate
 **
 ** Descriptions:		Gives the current data rate of connection to the
 ** 					associated wireless access point
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t Current data Rate
 **
 ***************************************************************************/
uint32_t wiflyGetWlanRate();

/*****************************************************************************
 ** Function name:		wiflyGetRSSI
 **
 ** Descriptions:		Prints current rssi value of wifly through
 ** 						the UART port
 **
 **
 ** parameters:			void
 *
 ** Returned value:		int16_t the obtained RSSI value
 **
 ****************************************************************************/
int16_t wiflyGetRSSI();

/*****************************************************************************
 ** Function name:		GetUptime
 **
 ** Descriptions:		Prints out through the UART port  the amount of time
 ** 						in which the device have remained powered on
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t Number of seconds that the wifly has been
 ** 								 powered on
 ** 								-1: in case of error
 **
 ****************************************************************************/
uint32_t wiflyGetUptime();

/*****************************************************************************
 ** Function name:		wiflyGetChannel
 **
 ** Descriptions:		Prints out through the UART port the wlan channel of
 ** 						the access point that it's currently associated
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t
 **
 ****************************************************************************/
uint32_t wiflyGetChannel();

/*****************************************************************************
 ** Function name:		wiflyGetBaudRate
 **
 ** Descriptions:		Function to ask the device for its current Baudrate
 ** 						value and print it through UART Debug Port
 **
 **
 ** parameters:			uint 8: void
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint32_t wiflyGetBaudRate();

/**************************************************************************
 ** Function name:		wiflyDisconnect
 **
 ** Descriptions:		Function to disconnect the device from the WLAN
 **
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint32_t wiflyDisconnect();

/*****************************************************************************
 ** Function name:		wiflyReboot
 **
 ** Descriptions:		Performs a reset on the device
 **
 **
 ** parameters:			void
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflyReboot();

/*****************************************************************************
 ** Function name:		wiflySaveConfig
 **
 ** Descriptions:		Function to store the configuration in the device's
 ** 						non-volatile memory
 **
 ****
 ** parameters:			void
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflySaveConfig();

/*****************************************************************************
 ** Function name:		wiflyjoinWlan
 **
 ** Descriptions: Function that forces to connect the device to a configured
 ** 				 wlan. If device was already connected the function forces to
 ** 				 re-connect
 **
 ** parameters:	 void
 **
 ** Returned value:		void
 **
 **
 ****************************************************************************/
void wiflyJoinWlan();

/*****************************************************************************
 ** Function name:		wiflySocketOpen
 **
 ** Descriptions: Function that opens the configured socket by sending the
 ** 			   command "open"
 **
 **
 ** parameters:	 void
 **
 ** Returned value:		uint8_t:
 ** 							1: if connection was successful
 ** 							0 otherwise
 **
 ****************************************************************************/
uint8_t wiflySocketOpen();

/*****************************************************************************
 ** Function name:		wiflySocketClose
 **
 ** Descriptions: Function that forces the disconnection of a
 ** 				socket by sending the comand "close"
 **
 **
 ** parameters:	 void
 **
 ** Returned value:
 ** 	uint8_t :1 If socket was properly closed (server replies *CLOSE*)
 ** 			 0	Otherwise
 **
 **
 ****************************************************************************/
uint8_t wiflySocketClose();

/*****************************************************************************
 ** Function name:		 wiflyRemoteConnectionSetup
 **
 ** Descriptions: Function that configure the wifly to connect to a generic
 ** 				 remote server
 **
 ** parameters:			uint8_t protocol: type of socket.
 ** 									HTTP,TCP client/server UDP etc.
 ** 					uint8_t remoteIp: The remote host ip address
 ** 						 				(0	if dns name is active)
 **
 ** 					uint8_t portRemote: remote socket port
 **
 ** 					char* host: In case of conection must be done
 ** 								by the server FQDN address
 **
 **
 ** Returned value:		uint8_t  1: If the configuration is successful
 ** 							 0: Otherwise
 **
 **
 ****************************************************************************/
uint8_t wiflyRemoteConnectionSetup(uint8_t protocol,char* remoteIp,uint16_t portRemote,uint16_t portLocal,char* host);

/*****************************************************************************
 ** Function name:		wiflyCommandtoServer
 **
 ** Descriptions: Prepares the command to be sent to the remote server
 **
 ** parameters: char* command: Command to make up
 **
 **
 ** Returned value: void
 **
 ****************************************************************************/
void wiflyCommandtoServer(char* command);

/*****************************************************************************
 ** Function name:		wiflyGetServerReply
 **
 ** Descriptions: Function to get the response from the remote server
 ** 				by listening to the UART port assigned to communicate
 ** 				to the device for a while
 **
 ** parameters: char* reply: pointer where the response must be stored
 ** 			uint32_t timewait: timeout for listening the response
 **
 ** Returned value: void
 **
 ****************************************************************************/
void wiflyGetServerReply(char* reply,uint32_t timeWait);

/*****************************************************************************
 ** Function name:		wiflyIsConnected
 **
 ** Descriptions: Function that checks whether the wifly is associated
 ** 				 to any WLAN or not
 ** parameters:	 void
 **
 **
 ** Returned value:		uint32_t  1: If device is associated
 ** 								  0: Otherwise
 **
 **
 ****************************************************************************/
 uint8_t wiflyIsConnected();




#endif /* WIFLY_H_ */
