 /***********************************************************************
 * debug.c
 *
 *  Created on: 16/10/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	debug.c
 * Description:
 * 		This is module provides with debug functionalities to
 *      help on finding problems by sending messages with
 *      timestamp through an UART port
 *
 ***********************************************************************/

 
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"
#include "LPC17xx.h"
#include "semphr.h"
#include "uart.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>
#include "utils.h"
#include "uprintf.h"
#include "TFC_Library_Setup.h"

/* Semaphore Definition*/
 static xSemaphoreHandle xMutexDebug=NULL;

 /*Definiton of Maximumm message length*/
#define BUFDEBUG		0xFF

 /*Storage for uart used for debugging purposes once it has been initialized*/
static uint32_t g_debug_port;

 /*Buffer to store the data received on the uart port */
 char g_debugger[BUFDEBUG];


/*************************PRIVATE FUNCTIONS DEFINITIONS*************/
 void _debugLock();
 void _debugUnlock();
/*******************************************************************/


/*****************************************************************************
 * Function name: debugLock();
 *
 * Descriptions:	Performs UART lock to prevent another instance to use
 * 					the resource
 *
 * parameters: none
 *
 * Returned value: void
 *
 *****************************************************************************/
 void _debugLock(){
	 if ( xMutexDebug==NULL){
		 xMutexDebug=xSemaphoreCreateRecursiveMutex();
	 }
	 xSemaphoreTakeRecursive( xMutexDebug,portMAX_DELAY);
 }

/*****************************************************************************
 * Function name: udebugfUnLock();
 *
 * Descriptions:	Performs UART lock to prevent another instance to use
 * 					the resource
 *
 * parameters: none
 *
 * Returned value: void
 *
 *****************************************************************************/
 void _debugUnlock(){
	xSemaphoreGiveRecursive( xMutexDebug);
}

/*****************************************************************************
* Function name: debugInit
*
* Descriptions:Initializes the UART port that will be used as Debugger
*
* parameters:
* 		uartPort: The selected UART port
* 		baudrate: The Baudrate to setup the communication
*
* Returned value: 1 if it manages to initialize the UART port properly
*
*****************************************************************************/
 uint32_t debugInit(uint8_t uartPort,uint32_t baudrate){
	 _debugLock();
	 uint8_t port=uartPort;
	 uint32_t init=0;
	 /* If selected UART is 2 or >3 the function initializes the UART0 by default */
	 if (uartPort==2||uartPort>3){
		 port=0;
	 }
	 init= UARTInit((uint32_t)port,baudrate);
	 UARTCleanPort(port);
	 g_debug_port=port;
	 _debugUnlock();
	 return init;
 }

/*****************************************************************************
 * Function name: debugPrintLog();
 *
 * Descriptions:	Sends through the debug Uart the values passed to
 * 					the function in same format as printf incluing a CR+LF at
 * 					the end
 *
 * parameters:		char* chain: pointer to the values passed to the function
 * 					formatted as in the same way as printf function
 *
 * Returned value: void
 *
 *****************************************************************************/
 void debugPrintLog(char *chain, ...){
	 _debugLock();
	 char* now=utilsGetTime();
	 char* header="***LOG***";
	 static char buffer[BUFDEBUG];
	 va_list args;
	 va_start(args,chain);
	 vsprintf(buffer,chain,args);
	 va_end(args);
	 sprintf(g_debugger,"%s %s \r\n",header,buffer);
	 UARTSend(g_debug_port,(uint8_t*)g_debugger,strlen(g_debugger));
	 FREEMEM(buffer,sizeof(char)*strlen(buffer));
	 FREEMEM(g_debugger,sizeof(char)*strlen(g_debugger));
	 UARTCleanPort(g_debug_port);
	 _debugUnlock();
 }


