 /***********************************************************************
 * motor.c
 *
 *  Created on: 24/11/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	motor.c
 * Description:
 * 			This library provides with funtions to manage
 *			the dual motor driver Qik2s9v1 and some other
 * 			funcionalities of Vilma's prototype
 *
 ***********************************************************************
 
 
/* Includes for LPC176X y FreRTOS libraries*/
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"
#include "LPC17xx.h"
/*Includes for UART port, wifly driver and other stuff*/
#include "TFC_Library_Setup.h"
#include "utils.h"

#include "motor.h"
#include "type.h"
#include "uart.h"
#include "uprintf.h"
#include "semphr.h"
#include "task.h"
#include "wifly.h"
#include "debug.h"
#include "type.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#define MOTOR_COMMAND_BUFFER				( 5 )
static uint8_t g_command [MOTOR_COMMAND_BUFFER];
static uint32_t g_motorUart = 99999;
const uint32_t g_uartTimeOut =1;
static uint8_t g_moving=0;
static uint8_t g_turning=0;


/* ***************************************************************
 * Structure movements vector definition
 * This is vector of movements data structure defined in motor.h
 * It is used to store the last MAXELEMENTS commands sent to the
 * vehicle.
 * **************************************************************/
movements mov[MAXMOVEMENTS];

/*This variable is used to control the current element stored int the
 * movements structure vector
 */
static uint8_t currentMovement=0;

/*Semaphore to prevent concurrent access to the device*/
static xSemaphoreHandle xMutexMotor=NULL;


/************************* Private Functions ************************************/

static void _motorLock();
static void _motorUnLock();
static void _motorReset();
static void _motorLeft();
static void _motorRight();
static void _motorStraighten();
static void _motorCleanBuffer();
static void _motorStoreMovement(uint16_t direction,uint16_t turn,uint8_t sp, uint8_t time);
static void _motorFW(uint8_t speed,uint8_t time);
static void _motorBW(uint8_t speed,uint8_t time);
/*******************************************************************************/


/*****************************************************************************
 * Function name: _motorLock
 *
 * Descriptions:Block the device to avoid any other instance uses it while
 * 				it is in use.
 *
 * parameters:			none
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _motorLock(){
	if(xMutexMotor==NULL){
		xMutexMotor=xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(xMutexMotor,portMAX_DELAY);
}

/*****************************************************************************
 * Function name: _motorUnlock
 *
 * Descriptions:Releases the device allowing other instances can lock it down
 *
 * parameters:			none
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _motorUnLock(){
	xSemaphoreGiveRecursive(xMutexMotor);
}

/*****************************************************************************
** Function name:		_motorReset
**
** Descriptions:		Function that performs a Hardware reset
** 						by setting the port P0.21 of LPC1769 (J6-23) to 0 for
** 						1sec
**
** @parameters:			void
** @Return:				void
**
*****************************************************************************/
static void _motorReset(){

	_motorLock();
	LPC_PINCON->PINSEL1 &= ( ~( 3 << 10 ) );
	LPC_GPIO0->FIODIR |= ( 1 << MOTOR_RESET_PIN	 );
	LPC_GPIO0->FIOCLR = 1 << MOTOR_RESET_PIN;
	utilsActiveDelay(1000);
	LPC_GPIO0->FIOSET = 1 << MOTOR_RESET_PIN;
	_motorUnLock();

}

/*****************************************************************************
** Function name:		_motorLeft
**
** @Descriptions:		Set the motor 1 (TURNMOTOR) to performs a turn to the left
** 						of the steering wheel
**
**
** @parameters:			void
**
** @Returned value:		void
**
*****************************************************************************/
 void _motorLeft(){
	_motorLock();
	uint8_t turn=100;   //setup the spin speed of the motor
	if(g_turning){
		_motorStraighten();
	}
	g_command[0] = TURNMOTOR_LEFT_CMD;
	g_command[1] = turn;
	UARTSend(g_motorUart,g_command, 2);
	_motorCleanBuffer();
	g_turning=1;
	_motorUnLock();
}

/*****************************************************************************
** Function name:		_motorRight
**
** @Descriptions:		Set the motor 1 (TURNMOTOR) to performs a turn to the
** 						right of the steering wheel
**
**
** @parameters:			void
**
** @Returned value:		void
**
*****************************************************************************/
 void _motorRight(){
	_motorLock();
	uint8_t turn=100;   //setup the spin speed of the motor
	if(g_turning){
		_motorStraighten();
	}
	g_command[0] = TURNMOTOR_RIGHT_CMD;
	g_command[1] = turn;
	UARTSend(g_motorUart,g_command, 2);
	_motorCleanBuffer();
	g_turning=1;
	_motorUnLock();
}

/*****************************************************************************
** Function name:		_motorStraighten
**
** Descriptions:	Set the motor 1 (TURNMOTOR) output to coast (High Z)
** 					making that motor to straighten the wheels
**
** @parameters:			void
** @Returned value:		void
**
*****************************************************************************/
static void _motorStraighten(){
	_motorLock();
	g_command[0]=TURNMOTOR_STRAIGHTEN_CMD;
	UARTSend(g_motorUart,g_command, 1);
	_motorCleanBuffer();
	g_turning=0;
	_motorUnLock();
}

/*****************************************************************************
** Function name:		_motorCleanBuffer
**
** Descriptions:	Resets the buffer storage used to send commands to the
** 					motor driver through the UART port
**
** @parameters:			void
** @Returned value:		void
**
*****************************************************************************/
static void _motorCleanBuffer(){
	int i=0;
	for (i = 0; i < MOTOR_COMMAND_BUFFER; ++i) {
		g_command[i]=0;
	}
}

/*****************************************************************************
 * Function name: _motorstoreMovement
 *
 * Descriptions: This function stores in a structure vector the latest 10
 * 				movements of the device. Once the 10 commands are completed
 * 				the function will override the values starting by the first
 * 				one
 *
 * parameters:			char direction: If it's forward or backward movement
 * 						char turn: If it's left or right movement
 * 						uint8_t The speed sent by the command
 * 						uint8_t The time that command must be in execution
 *
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _motorStoreMovement(uint16_t direction,uint16_t turn,uint8_t sp, uint8_t time){

	/*Definition of structures movementDefinition vector to store up
	 to MAXMOVEMENTS elements. This values is defined in motor.h*/
	if(currentMovement>=MAXMOVEMENTS){
		currentMovement=0;
	}
	mov[currentMovement].move=direction;
	mov[currentMovement].turn=turn;
	mov[currentMovement].speed=sp;
	mov[currentMovement].time=time;
	currentMovement++ ;

}

/*****************************************************************************
** Function name:		_motorFW
** **
** @Descriptions:		This function implements the same operations as motorForward
** 						but without saving the momenment into the tracker. It only to
** 						be used as composed functions with movements and turns such as
** 						motorForwardRight and Left
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
static void _motorFW(uint8_t speed,uint8_t time)
{
	_motorLock();
	if (speed >60){
		speed = 60;
	}
	g_command[0] = DIRMOTOR_FORWARD_CMD;
	g_command[1] = speed;
	UARTSend(g_motorUart,g_command, 2);
	utilsPassiveDelay(time*1000);
	motorStop();
	_motorCleanBuffer();
	_motorReset();
	g_command[0]=0;
	_motorUnLock();
}

/*****************************************************************************
** Function name:		_motorBW
** **
** @Descriptions:		This function implements the same operations as motorForward
** 						but without saving the movements into the tracker. It only to
** 						be used as composed functions with movements and turns such as
** 						motorBackwardRight and Left
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
static void _motorBW (uint8_t speed,uint8_t time)
{
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	g_command[0] = DIRMOTOR_BACKWARD_CMD;
	g_command[1] = speed;
	UARTSend(g_motorUart, g_command, 2);
	utilsPassiveDelay(time*1000);
	motorStop();
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();
}

/***************************** PUBLIC FUNCTIONS *******************************/




/*****************************************************************************
** Function name:		motorInit
**
** Descriptions:		Initializes the uart port and the baudRate
** 						to talk to the motor driver and sends the command a
** 						MOTOR_INIT_CMD to ends up with the baudrate autodetection
** 						doing by default by the driver
**
**
** @parameters:			uart: UART assigned to communicate to the device
**
** @Return:		1 if init was successful
** 				0 Otherwise
**
*****************************************************************************/
uint8_t motorInit(uint32_t uart)
{
	_motorLock();
	uint8_t ret = FALSE;
	if (uart >= 0||uart<=2)
	{
		g_motorUart=uart;
		ret=UARTInit(g_motorUart,MOTOR_BAUDRATE); // init uart at 38400bps
		_motorReset();
		utilsActiveDelay(1000);
		g_command[0] = MOTOR_INIT_CMD;  // Init baudrate: autodetect
		UARTSend(g_motorUart, g_command,1);
		_motorCleanBuffer();
		_motorUnLock();
		return ret;
	}
}

/*****************************************************************************
** Function name:		motorStop
**
** Descriptions:	Set the motor 0 (DIRMOTOR) output to coast (High Z)
** 					stopping the motor
**
** @parameters:			void
** @Returned value:		void
**
*****************************************************************************/
void motorStop()
{
	_motorLock();
	if(g_turning){
		_motorStraighten();
	}
	g_command[0]=DIRMOTOR_STOP_CMD;
	UARTSend(g_motorUart,g_command, 1);
	g_command[0]=0;
	utilsPassiveDelay(1000);
	g_moving=0;
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();

}

/*****************************************************************************
** Function name:		motorForward
**
** @Descriptions:		Set the DIRMOTOR to spin Forward with the given speed
** 						and for the time passed as a parameter.
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorForward(uint8_t speed,uint8_t time)
{
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	g_command[0] = DIRMOTOR_FORWARD_CMD;
	g_command[1] = speed;
	UARTSend(g_motorUart,g_command, 2);
	utilsPassiveDelay(time*1000);
	_motorStoreMovement(DIRMOTOR_FORWARD_CMD,0,speed,time);
	motorStop();
	_motorCleanBuffer();
	_motorReset();
	g_command[0]=0;
	_motorUnLock();
}

/*****************************************************************************
** Function name:		motorBackward
**
** @Descriptions:		Set the DIRMOTOR (M0) to spin Backwards with the given speed
** 						and for the time passed as a parameter.
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorBackward(uint8_t speed,uint8_t time)
{
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	g_command[0] = DIRMOTOR_BACKWARD_CMD;
	g_command[1] = speed;
	UARTSend(g_motorUart, g_command, 2);
	utilsPassiveDelay(time*1000);
	_motorStoreMovement(DIRMOTOR_BACKWARD_CMD,0,speed,time);
	motorStop();
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();
}

/*****************************************************************************
** Function name:		motorForwardLeft
**
** @Descriptions:		Set the TURNMOTOR (M1) to turns to the left and the DIRMOTOR
** 					    to go forward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped)
** 						to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void  motorForwardLeft(uint8_t speed,uint8_t time){
	_motorLock();
	if (speed > 60){
			speed = 60;
		}
		_motorLeft();
		_motorFW(speed,time);
		_motorStoreMovement(DIRMOTOR_FORWARD_CMD,TURNMOTOR_LEFT_CMD,speed,time);
		_motorCleanBuffer();
		_motorReset();
		_motorUnLock();
}

/*****************************************************************************
** Function name:		motorForwardRight
**
** @Descriptions:		Set the TURNMOTOR to turns to the right and the DIRMOTOR
** 					    to go forward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stopped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorForwardRight(uint8_t speed,uint8_t time){
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	_motorRight();
	_motorFW(speed,time);
	_motorStoreMovement(DIRMOTOR_FORWARD_CMD,TURNMOTOR_RIGHT_CMD,speed,time);
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();
}

/*****************************************************************************
** Function name:		motorBackwardRight
**
** @Descriptions:		Set the TURNMOTOR to turns to the right and the DIRMOTOR
** 					    to go backward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void  motorBackwardRight(uint8_t speed,uint8_t time){
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	_motorRight();
	_motorBW(speed,time);
	_motorStoreMovement(DIRMOTOR_BACKWARD_CMD,TURNMOTOR_RIGHT_CMD,speed,time);
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();
}

/*****************************************************************************
** Function name:		motorBackwardLeft
**
** @Descriptions:		Set the TURNMOTOR to turns to the left and the DIRMOTOR
** 					    to go backward for the time and speed passed as parameter
**
**
** @parameters:			uint8_t speed: Speed Value from 0 (stoped) to 127 (full speed)
** 						uint8_t time: seconds of movement
**
** @Returned value:		void
**
*****************************************************************************/
void motorBackwardLeft(uint8_t speed,uint8_t time){
	_motorLock();
	if (speed > 60){
		speed = 60;
	}
	_motorLeft();
	_motorBW(speed,time);
	_motorStoreMovement(DIRMOTOR_BACKWARD_CMD,TURNMOTOR_LEFT_CMD,speed,time);
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();
}


/*****************************************************************************
 * Function name: motorUndoMovement
 *
 * Descriptions: This function gets the last command stored into the vector
 * 				movements and executes the previous movement stored
 *
 *
 * parameters:			void
 *  Returned value:		void
 *****************************************************************************/
void motorUndoMovement(){
	_motorLock();
	uint8_t prevMovement=-1;
	uint16_t lastDirection;
	uint16_t lastTurn;
	uint8_t lastSpeed;
	uint8_t lastTime;
	char* message;
	char* l_movement;
	char* l_turn;
	//Previous movement is numElement-1 except when the last movement is stored
	// into the last vector position which is MAXELEMENTS-1
	//
	if(currentMovement==0){
		prevMovement=MAXMOVEMENTS-1;
	}else{
		prevMovement=currentMovement-1;
	}
	lastDirection=mov[prevMovement].move; //takes value to know if last movement was forward or backward
	lastTurn=mov[prevMovement].turn;	//take the value to know if it was left,right or none
	lastSpeed=mov[prevMovement].speed;
	lastTime=mov[prevMovement].time;
	currentMovement=prevMovement;
	motorStop();
	switch (lastDirection) {
	case DIRMOTOR_FORWARD_CMD:
		l_movement="BACKWARDS";
		switch (lastTurn) {
		case TURNMOTOR_LEFT_CMD:
			l_turn="RIGHT";
			_motorStraighten();
			_motorRight();
			_motorBW(lastSpeed,lastTime);
			_motorStoreMovement(DIRMOTOR_BACKWARD_CMD,TURNMOTOR_RIGHT_CMD,lastSpeed,lastTime);
			break;
		case TURNMOTOR_RIGHT_CMD:
			l_turn="LEFT";
			_motorStraighten();
			_motorLeft();
			_motorBW(lastSpeed,lastTime);
			_motorStoreMovement(DIRMOTOR_BACKWARD_CMD,TURNMOTOR_LEFT_CMD,lastSpeed,lastTime);
			break;
		default:
			motorBackward(lastSpeed,lastTime);
			break;
		}
		break;
		case DIRMOTOR_BACKWARD_CMD:
			l_movement="FORWARD";
			switch (lastTurn) {
			case TURNMOTOR_LEFT_CMD:
				l_turn="RIGHT";
				_motorStraighten();
				_motorRight();
				_motorFW(lastSpeed,lastTime);
				_motorStoreMovement(DIRMOTOR_FORWARD_CMD,TURNMOTOR_RIGHT_CMD,lastSpeed,lastTime);
				break;
			case TURNMOTOR_RIGHT_CMD:
				l_turn="LEFT";
				_motorStraighten();
				_motorLeft();
				_motorFW(lastSpeed,lastTime);
				_motorStoreMovement(DIRMOTOR_FORWARD_CMD,TURNMOTOR_LEFT_CMD,lastSpeed,lastTime);
				break;
			default:
				motorForward(lastSpeed,lastTime);
				break;
			}

			default:
				motorStop();
				break;
	}
	if(lastTurn==0){
		message=utilsMakeChain("Undoing movement:Vilma went to:%s",l_movement);
	}else{
		message=utilsMakeChain("Undoing movement:Vilma went to: %s %s",l_movement, l_turn);
	}

	if(DEBUGGER){
		printf(message);
		debugPrintLog(message);
	}
	_motorUnLock();
}

/*****************************************************************************
 * Function name: motoTests
 *
 * Descriptions: This function executes motor turn tests on both motors to make sure
 * 				 they are working and properly connected
 *
 * 	
 *
 * parameters:			void
 *  Returned value:		void
 *****************************************************************************/
void motorTests(){
	_motorLock();
	uint8_t initSpeed=30;
	uint8_t testTime=4;
	debugPrintLog("testing motors...\r\n");
	debugPrintLog("Movement Forward Left\r\n");
	motorForwardLeft(initSpeed,testTime);
	debugPrintLog("Movement Forward Right\r\n");
	motorForwardRight(initSpeed,testTime);
	debugPrintLog("Movement Backwards Left\r\n");
	motorBackwardLeft(initSpeed,testTime);
	debugPrintLog("Movement Backwards Right\r\n");
	motorBackwardRight(initSpeed,testTime);
	_motorCleanBuffer();
	_motorReset();
	_motorUnLock();

}



