/*
 * ping.c
 *
 *  Created on: 28/12/2014
 *      Author: Javier
 */
 /***********************************************************************
 * ping.c
 *
 *  Created on: 28/12/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	ping.c
 * Description:
 * 			This library contains functions to get de distance either
 * 			in centimeters or in inches from the voltage value given by the
 *			ultrasonic sensor MaxSonar-EZ0.
 * 			It uses the ADC driver to read the escaling value and
 * 			makes the conversion form milliVolts to inches/centimeters 
 * 			
 *
 ***********************************************************************/
#include "FreeRTOS.h"
#include "LPC17xx.h"
#include "semphr.h"
#include "type.h"
#include "adc.h"
#include "ping.h"
#include "utils.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

/*ADC Library setup*/
extern volatile uint32_t ADCValue[ADC_NUM];
extern volatile uint32_t ADCIntDone;
extern void TargetResetInit(void);
#define ADC_COUNT_MAX (4095)
#define ULTRASONIC_CONSTANT (512)

/*Semaphore to prevent concurrent access to the device*/
xSemaphoreHandle xMutexPing=NULL;



static uint8_t g_adcChannel;
static uint32_t ultrasonicValue;
static double ultrasonicCm;


/*****************************************************************************
 * Function name: _pingLock
 *
 * Descriptions:Block the device to avoid any other instance uses it while
 * 				it is in use.
 *
 * parameters:			none
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _pingLock(){
	if(xMutexPing==NULL){
		xMutexPing=xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(xMutexPing,portMAX_DELAY);

}

/*****************************************************************************
 * Function name: _pingUnlock
 *
 * Descriptions:Releases the device allowing other instances can lock it down
 *
 * parameters:			none
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _pingUnLock(){
	xSemaphoreGiveRecursive(xMutexPing);
}



/*****************************************************************************
 * Function name: pingInit
 *
 * Descriptions:Initializes the selected ADC channel and clk to be used
 *
 * parameters:			channel: The ADC channel to initialize
 *
 * Returned value:		void
 *
 *****************************************************************************/
void pingInit(uint8_t channel){
	_pingLock();
	if (channel>ADC_NUM){
		g_adcChannel=0;
	}else{
		g_adcChannel=channel;
	}
	ADCInit(ADC_CLK);
	_pingUnLock();
}

/*****************************************************************************
 * Function name: pingGetDistanceInches
 *
 * Descriptions:Reads the ADC voltage value and performs a conversion
 * 				volts-inches given by the manufacturer of sensor
 * 				LV-MaxSonar-EZ0.
 *
 * 			 	Function: value/ADC_COUNT_MAX)*ULTRASONIC_CONSTANT, where
 *
 * 			 	value: Voltage obtained on ping AN of sensor
 * 			 	ADC_COUNT_MAX: Maximum scaling level on LPC1769, since ADC
 * 			 	has 12 bits	(2^12=4096) from 0 to 4095
 * 			 	ULTRASONIC_CONSTANT: scaling constant given by manufatrurer
 *
 *
 * parameters:			none
 *
 * Returned value:		double: The distance in inches
 *
 *****************************************************************************/
float pingGetDistanceInches(){
	_pingLock();
	float inches,value;
	const portTickType readDelay=125/portTICK_RATE_MS;
	value=ADCRead(g_adcChannel);
	inches=(value/ADC_COUNT_MAX)*ULTRASONIC_CONSTANT;
	_pingUnLock();
	return inches;
	}

/*****************************************************************************
 * Function name: pingGetDistanceCm
 *
 * Descriptions:Gets the distance in inches and does the conversion
 * 				from inches to centimeters.
 *
 * parameters:
 *
 * Returned value:		double: The distance in centimeters
 *
 *****************************************************************************/
float pingGetDistanceCm(){
	const double cmPerInch=0.3937f;
	float cm;
	cm=pingGetDistanceInches()/cmPerInch;
	return cm;
}




