/***********************************************************************
 *
 *
 *  Created on: 12/10/2014
 *      Author: javier
 *
 * Project:	uprintf
 * File:	uprintf.c
 * Description:
 * 			This library implements the appliances to send and receive
 * 			data through the UART ports in a proper text format.
 *
 ***********************************************************************/

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"

#include "task.h"
#include "semphr.h"
#include "uart.h"
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "utils.h"
#include "uprintf.h"
#include "TFC_Library_Setup.h"

#define UPRINTFBUFSIZE 0xFF
/* Semaphore Definition*/
 static xSemaphoreHandle xMutexPrint=NULL;
 static uint32_t g_uprintf_port;

 /*Buffer to store the data received on the uart port */
 char uprintfBuffer	 [UPRINTFBUFSIZE];


 /*************************PRIVATE FUNCTIONS DEFINITIONS*************/

 void _uprintfLock();
 void _uprintfUnlock();

 /*******************************************************************/




 /*****************************************************************************
 * Function name: uprintfLock();
 *
 * Descriptions:Performs UART lock to prevent another instance to use the resource
 *
 * parameters: none
 *
 * Returned value: void
 *
 *****************************************************************************/

 void _uprintfLock()
 {

	 if (xMutexPrint==NULL){
		 xMutexPrint=xSemaphoreCreateRecursiveMutex();
	 }
	 xSemaphoreTakeRecursive(xMutexPrint,portMAX_DELAY);
 }


  /*****************************************************************************
  * Function name: uprintfUnlock(){
  *
  * Descriptions:	Releases the mutual exclusion of the resource
  *
  * parameters: none
  *
  * Returned value:void
  *
  *****************************************************************************/

 void _uprintfUnlock(){
  	xSemaphoreGive(xMutexPrint);
  }


/*****************************************************************************
* Function name: uprintInit
*
* Descriptions:Initializes the UART port that will be used for sending and
* 			   receiving data
*
* parameters:
* 		uartPort: The selected UART port
* 		baudrate: The Baudrate to setup the communication
*
* Returned value: 1 if it manages to initialize the UART port properly
*
*****************************************************************************/
 uint32_t uprintfInit(uint8_t uartPort,uint32_t baudrate){
	 _uprintfLock();
	 uint8_t port=uartPort;
	 uint32_t init=0;

	 /* If selected UART is 2 or >3 the function initializes the UART0 by default */
	 if (uartPort==2||uartPort>3){
		 port=0;
	 }
	 init= UARTInit((uint8_t)port,baudrate);
	 UARTCleanPort(port);
	 g_uprintf_port=port;
	_uprintfUnlock();
	 return init;

 }


/*****************************************************************************
 * Function name: uprintfScanf
 *
 * Descriptions:The function reads the selected UART Buffer for the milliseconds
 * 			   passed as parameter
 *
 * parameters:
 *
 * 			response: pointer to the text received by UART
 * 			timeout: time wait for more data received
 *
 * Returned value:
 *
 *****************************************************************************/
uint32_t uprintfScanf(char *response, uint32_t timeout){
	_uprintfLock();
	uint32_t reply=0;
	reply=UARTScanf(g_uprintf_port,uprintfBuffer,timeout);
	_uprintfUnlock();
return reply;
}


/*****************************************************************************
 * Function name: uprintfSend();
 *
 * Descriptions:	Sends the through the initialized Uart the valued passed to
 * 					the function in same format as printf
 *
 * parameters:		char* chain: pointer to the values passed to the function
 * 					formatted as in the same way as printf function
 *
 *
 *
 * Returned value: void
 *
 *****************************************************************************/
void uprintfSend(const char *chain, ...){
	_uprintfLock();
	UARTCleanPort(g_uprintf_port);
	va_list args;
	va_start(args,chain);
	vsprintf(uprintfBuffer,chain,args);
	va_end(args);
	UARTSend(g_uprintf_port,(uint8_t*)uprintfBuffer,strlen(uprintfBuffer));
	utilsFreeMem(uprintfBuffer);
	_uprintfUnlock();
}


/*****************************************************************************
 * Function name: uprintfSendLn();
 *
 * Descriptions:	It provides the same functionality as printfSendUart function
 * 					but also adds a CR+LF code to returning the line
 *
 * parameters:		char* chain: pointer to the values passed to the function
 * 					formatted as in the same way as printf function
 *
 *
 * Returned value: void
 *
 *****************************************************************************/
void uprintfSendLn(const char *chain, ...){
	_uprintfLock();
	va_list args;
	va_start(args,chain);
	vsprintf(uprintfBuffer,chain,args);
	va_end(args);
	strcat(uprintfBuffer,"\r\n");
	UARTSend(g_uprintf_port,(uint8_t*)uprintfBuffer,strlen(uprintfBuffer));
	FREEMEM(uprintfBuffer,sizeof(char)*strlen(uprintfBuffer));
	UARTCleanPort(g_uprintf_port);
	_uprintfUnlock();

}








