 /***********************************************************************
 * vilmaControl.h
 *
 *  Created on: 27/11/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	utils.h
 * Description:
 * 			This library implements the most important functionalities
 *			of Vilma's project. Such as get, proccess and execute remote			
 *			commands.
 *
 ***********************************************************************

/*************************** BUFFER LENGTH *********************************************************/
#define BUFSIZE		0xFF
#define SHORT_BUFFER 0x1A
/***************************************************************************************************/
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"
#include "semphr.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "utils.h"
#include "uprintf.h"
#include "debug.h"
#include "wifly.h"
#include "motor.h"
#include "TFC_Library_Setup.h"
#include "ping.h"
#include "vilmaControl.h"

/*Variable to store the messages received through the TCP socket*/
static char g_tcpMessage[BUFSIZE];


/*Variable to store the status of the vehicle*/
static char g_vilmaSSID[SHORT_BUFFER];
static char g_vilmaIp[SHORT_BUFFER];
static char g_command[SHORT_BUFFER];

extern volatile int16_t g_rssiThreshold;
extern volatile int16_t g_currentRssi;
extern volatile uint32_t g_currentDistance;

/*Semaphore to prevent concurrent access to a device*/
static xSemaphoreHandle xMutexControl=NULL;


 char* serverSend="GET$/status.php?";



/******************************** PROTOTYPES OF PRIVATE FUNCTIONS***********************************/

static void _vilmaControlLock();
static void _vilmaControlUnLock();
static void _vilmaControlSendTracker(char* message);
static void _vimaControlSetMinRSSI(int16_t mRssi);
static  void _helloRemote();


/***************************************************************************************************/


/************************************ PRIVATE FUNCTIONS********************************************/



/******************************************************************************************************
 * Function name: vilmaControlLock
 *
 * Descriptions:Mutex to protect against any other function interferes with the one in execution
 *
 *
 * parameters:
 * 				void
 *
 * Returned value:
 * 				void
 *
 ******************************************************************************************************/
static void _vilmaControlLock(){
	if (xMutexControl==NULL){
		xMutexControl=xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(xMutexControl,portMAX_DELAY);
}
/******************************************************************************************************
 * Function name: vilmaControlUnLock
 *
 * Descriptions:Releases de Mutex
 *
 * parameters:
 * 				void
 *
 * Returned value:
 * 				void
 *
 ******************************************************************************************************/
static void _vilmaControlUnLock(){
	xSemaphoreGiveRecursive(xMutexControl);
}


/******************************************************************************************************
 * Function name: vilmaControlHelloServer
 *
 * Descriptions:Sends to the remote peer the information of initial status of wifly device
 * 					 such as:
 * 					 wifly's ip and listen port,
 * 					 rssi level and rssi threshold
 * 					 distance of nearest front object
 *
 * parameters:
 * 				void
 *
 * Returned value:
 * 				void
 *
 ******************************************************************************************************/
 void _vilmaControlHelloServer(){
	_vilmaControlLock();
	 char* message1;
	char* message2;
	char* msg="Vilma_Ready_to_Rock!";
	uint16_t port;
	port=wiflyGetPort();
	message1=utilsMakeChain("vP=%i&stW=%s&vIp=%s",port,g_vilmaSSID,g_vilmaIp);
	utilsPassiveDelay(200);
	vilmaControlSendSocketData(message1);
	message2=utilsMakeChain("msg=%s&dst=%i&rssi=%i&mRs=%i",msg,g_currentDistance,g_currentRssi,g_rssiThreshold);
	utilsPassiveDelay(200);
	vilmaControlSendSocketData(message2);
	_vilmaControlUnLock();

 }

 /******************************************************************************************************
  * Function name: _vilmaControlSendTracker
  *
  * Descriptions:Send to the remote server the last movement performed by the vehicle for it to print
  * 			the tracking information
  *
  * parameters:
  * 				void
  *
  * Returned value:
  * 				void
  *
  ******************************************************************************************************/
 static void _vilmaControlSendTracker(char* message){
	 _vilmaControlLock();
	  FREEMEM(g_tcpMessage,sizeof(char)*strlen(g_tcpMessage));
	  strcpy(g_tcpMessage,serverSend);
	  strcat(g_tcpMessage,message);
	 wiflyCommandtoServer(g_tcpMessage);
	 FREEMEM(g_tcpMessage,sizeof(char)*strlen(g_tcpMessage));
	 _vilmaControlUnLock();
 }

 /******************************************************************************************************
  * Function name: _vilmaControlSendStatus
  *
  * Descriptions: Function that sends the current status of the values rssi, minumum RSSI and distance
  * 			  in order the system could be aware of if there is a lack of signal or if vehicle is
  * 			  about to hit any object
  *
  * parameters:
  * 				void
  *
  * Returned value:
  * 				void
  *
  ******************************************************************************************************/
 void _vilmaControlSendStatus(){
	_vilmaControlLock();
	char* msg="Vilma_Ready_to_Rock!";
	g_currentDistance=pingGetDistanceCm();
	char* status=utilsMakeChain("msg=%s&rssi=%i&mRs=%i&dst=%i",msg,g_currentRssi,g_rssiThreshold,g_currentDistance);
	vilmaControlSendSocketData(status);
	_vilmaControlUnLock();
}

 /******************************************************************************************************
  * Function name: vilmaControlSetMinRSSI
  *
  * Descriptions: Function that keeps the value of minimum permitted rssi value updated
  *
  * parameters:
  * 			int16_t mRssi: The new rssi value
  *
  * Returned value:
  * 				void
  *
  ******************************************************************************************************/
 static void _vimaControlSetMinRSSI(int16_t mRssi){
	 g_rssiThreshold=mRssi;
 }


/***************************************************************************************************/


/************************************ PUBLIC FUNCTIONS**********************************************/


/******************************************************************************************************
 * Function name: vilmaControlRemoteInit
 *
 * Descriptions:Initialize the device wireless module on the Vehicle to be able to connect
 * 				via socket TCP to a remote peer
 *
 * parameters:
 * 			remoteIp: The ip address of the remote peer
 * 			remotePort:The port where remote peer will be listening
 * 			localPort: To setup where vilma will be listening for commands
 * 					port is 2000 by default
 * 			remoteHost: the FQDN of the remote peer in case of performing connection
 * 					through the its hostname in place of its ip address (if this configuration
 * 					is selected, the parameter "remoteIp" won't be used no matter if any value have
 * 					passed to the function)
 *
 *
 * Returned value:
 * 				1 if the the setup was successful
 * 				0 Otherwise
 *
 ******************************************************************************************************/
uint8_t vilmaControlRemoteInit (char* remoteIp,uint16_t remotePort,uint16_t localPort,char* remoteHostname){
	_vilmaControlLock();
	wiflyRemoteConnectionSetup(SOCKET_TCP,remoteIp,remotePort,localPort,remoteHostname);
	/*Getting vehicle's value to be sent to the remote application*/
	wiflyGetIp(g_vilmaIp);
	wiflyGetSSID(g_vilmaSSID);
	g_currentDistance=pingGetDistanceCm();
	g_currentRssi=wiflyGetRSSI();
	_vilmaControlHelloServer();
	_vilmaControlUnLock();
}


/******************************************************************************************************
 * Function name: vilmaControlGetSocketData
 *
 * Descriptions:Takes the command sent by the remote peer via the TCP socket connection
 *
 * parameters:
 * 			command: pointer where stores the complete data taken out from the socket
 *
 * Returned value:
 * 				char: pointer to data received on the socket
 *
 ******************************************************************************************************/
char* controlServerGetSocketData(){
	FREEMEM(g_tcpMessage,sizeof(char)*strlen(g_tcpMessage));
	_vilmaControlLock();
	char* data;
	char*chain=(char*)pvPortMalloc(sizeof(chain));
	char token[2]=";";
	wiflyGetServerReply(g_tcpMessage,TCP_LISTEN_TIMEOUT);
	if (utilsCheckValue(g_tcpMessage,token)){
		sprintf(chain,"%s",strstr(g_tcpMessage,token)+strlen(token));
		strtok(chain,token);
		strpbrk(chain,"\0");
		if(strlen(chain)!=0){
			data=chain;
		}
	}else {
		data=NULL;
	}
	if(DEBUGGER&&data!=NULL){
		debugPrintLog(data);
	}
	vPortFree(chain);
	_vilmaControlUnLock();
	return data;
}

/******************************************************************************************************
 * Function name: vilmaControlSendSocketData
 *
 * Descriptions:Sends data to the remote peer through the socket TCP
 *
 * parameters:
 * 			command: command to be sent
 * Returned value:
 * 				void
 *
 ******************************************************************************************************/

void vilmaControlSendSocketData(char* command){
	_vilmaControlLock();
	char l_message [BUFSIZE];
	sprintf(l_message,"%s%s ",serverSend,command);
	strcat(l_message,"\0");
	wiflyCommandtoServer(l_message);
	utilsPassiveDelay(200);
	FREEMEM(l_message,sizeof(char)*BUFSIZE);
	_vilmaControlUnLock();

}


/******************************************************************************************************
 * Function name: vilmaControlExecCommand
 *
 * Descriptions:Executes the command passed as parameter.
 *
 * parameters:
 * 			command: comand to execute
 * Returned value:
 * 				void
 *
 ******************************************************************************************************/
void vilmaControlExecCommand(char* command){
	_vilmaControlLock();
	uint8_t defTime=4;
	uint8_t defSpeed=32;
	int8_t rssi;
	uint8_t res=0;
	int8_t speed=0;
	int8_t time=0;
	char cmd[BUFSIZE];
	strcpy(cmd,command);
	char* ptr;
	char* words[5];
	char* token=",";
	char* mov;
	char* tracker;
	ptr=strtok(cmd,token);
	while(ptr!=NULL){
		words[res]=ptr;
		ptr=strtok(NULL,token);
		res++;
	}
	mov=words[0];
	speed=atoi(words[1]);
	time=atoi(words[2]);

	if (speed==0){
		speed=defSpeed;
	}
	if (time<defTime){
		time=defTime;
	}

	if(strcmp(mov,"RSSI")==0){
		_vimaControlSetMinRSSI(speed);//The minimum RSSI value comes into the speed variable
		tracker=utilsMakeChain("msg=Executed:Minium_RSSI_level_set_to_%i",g_rssiThreshold);
	}else if(strcmp(mov,"UNDO")==0){
		motorUndoMovement();
		tracker="msg=Executed:Undoing last movement";
	}else if(strcmp(mov,"F_LEFT")==0){
		motorForwardLeft(speed,time);
		tracker=utilsMakeChain("msg=Executed:FORWARD_LEFT-speed:%i-time:%i",speed,time);
	}else if (strcmp(mov,"FORWARD")==0) {
		motorForward(speed,time);
		tracker=utilsMakeChain("msg=Executed:FORWARD-speed:%i-time:%i",speed,time);
	}else if (strcmp(mov,"F_RIGHT")==0) {
		motorForwardRight(speed,time);
		tracker=utilsMakeChain("msg=Executed:FORWARD_RIGHT-speed:%i-time:%i",speed,time);
	}else if (strcmp(mov,"B_LEFT")==0) {
		motorBackwardLeft(speed,time);
		tracker=utilsMakeChain("msg=Executed:BACKWARD_LEFT-speed:%i-time:%i",speed,time);
	}else if (strcmp(mov,"BACKWARD")==0) {
		motorBackward(speed,time);
		tracker=utilsMakeChain("msg=Executed:BACKWARD-speed:%i-time:%i",speed,time);
	}else if (strcmp(mov,"B_RIGHT")==0) {
		motorBackwardRight(speed,time);
		tracker=utilsMakeChain("msg=Executed:BACKWARD_RIGHT-speed:%i-time:%i",speed,time);
	}else{
		motorStop();
		tracker=utilsMakeChain("msg=Executed:FULL_STOP");

	}
	_vilmaControlSendTracker(tracker);
	FREEMEM(cmd,sizeof(char)*BUFSIZE);
	_vilmaControlUnLock();
}

/******************************************************************************************************
 * Function name: vilmaControlGetMinRssi
 *
 * Descriptions:Gets the current configured minimum RSSI value permitted
 *
 * parameters: void
 *
 * Returned value:
 * 				int16_t: Minimum RSSI value
 *
 ******************************************************************************************************/
int16_t vilmaControlGetMinRssi(){

	return g_rssiThreshold;
}

/******************************************************************************************************
 * Function name: vilmaCotrolGetCurrentRssi
 *
 * Descriptions:Gets the current RSSI value
 *
 * parameters: void
 *
 * Returned value:
 * 				int16_t: Current RSSI value
 *
 ******************************************************************************************************/
int16_t vilmaCotrolGetCurrentRssi(){
	return g_currentRssi;
}

/******************************************************************************************************
 * Function name: vilmaControlGetCurrentDistance
 *
 * Descriptions:Gets the current distance upto the nearest object in front of the vehicle
 *
 * parameters: void
 *
 * Returned value:
 * 				int23_t: Current distance in centimeters
 *
 ******************************************************************************************************/
uint32_t vilmaControlGetCurrentDistance(){
	return g_currentDistance;
}




