 /***********************************************************************
 * wifly.h
 *
 *  Created on: 22/10/2014
 *      Author: javier
 *
 * Project:	TFC_Library
 * File:	wifly.h
 * Description:
 * 			This library contains functions to manage the 
 *			device RN171XV (Wifly) allowing its configuration as well as
 *			get important information about its state
 *
 ***********************************************************************/

#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "FreeRTOSConfig.h"
#include "semphr.h"
#include "uart.h"
#include "TFC_Library_Setup.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include "utils.h"
#include "uprintf.h"
#include "debug.h"
#include "wifly.h"
#include "task.h"



/*Variable that keeps the information about the UART to communicate to*/
static uint8_t g_uartWifly=-1;

/*variable to control whether the wifly is in command mode */
static uint8_t g_commandMode=0;

/*variable to control whether the wifly is in associated to any WLAN */
static uint8_t g_wiflyConected=0;

/*Variable to ask if remote server have already being setup*/
static uint8_t g_serverConfigured=0;

/*Semaphore for preventing concurrent access to the device*/
static xSemaphoreHandle xMutexWifly=NULL;

/*variable to store the response answered by the device after every message received */
static char g_wiflyReply	 [BUFSIZE];

/*variable to make up the commands to be sent to the device */
static char g_buffer[BUFSIZE];

/*Variable to store different kind of messages */
static char* g_message;




/******************** PROTOTYPES OF PRIVATE FUNCTIONS*********************/


static void _wiflyLock();
static void _wiflyUnlock();
static void _wiflyReset();
static void _wiflySetCommandMode();
static void _wiflySetNormalMode();
static uint8_t _wiflyCheckReply(char* g_buffer,char* pattern );
void _wiflySendUartLn(const char *chain, ...);
static void _wiflySendUart(const char *chain, ...);
static uint32_t _wiflyGetReply(char *response, uint32_t timeout);


/****************************************************************************/


/*****************************************************************************
 * Function name: _wiflyLock
 *
 * Descriptions:Block the device to avoid any other instance uses the device while
 * 				it is in use.
 *
 * parameters:			none
 *
 * Returned value:		void
 *
 *****************************************************************************/
static void _wiflyLock(){
	if (xMutexWifly==NULL){
		xMutexWifly=xSemaphoreCreateRecursiveMutex();
	}
	xSemaphoreTakeRecursive(xMutexWifly,portMAX_DELAY);
}


/*****************************************************************************
 * Function name: _wiflyUnlock
 *
 * Descriptions:Releases the device allowing another instance can use the device
 *
 *
 * parameters:
 * 			none
 *
 * Returned value:
 * 				void
 *
 *****************************************************************************/
static void _wiflyUnlock(){
	xSemaphoreGiveRecursive(xMutexWifly);
}

/*****************************************************************************
 * Function name: _wiflyReset
 *
 * Descriptions:Performs a hardware reset on Wifly device placing 0-1 sequence on
 *              its pin 5 for a period of 1ms by using port address P0_22
 *              (lpc1679 board pin J6-24)
 * parameters:
 * 			none
 *
 * Returned value:
 * 				void
 *
 *****************************************************************************/
static void _wiflyReset(){
	_wiflyLock();
	//Inializes P0.22
	LPC_PINCON->PINSEL1 &= ( ~( 3<<12) );
	LPC_GPIO0->FIODIR |= ( 1 << WF_RESET_PIN );
	utilsActiveDelay(1);
	//Setting up the port in low level for 1ms
	LPC_GPIO0->FIOCLR=1<<WF_RESET_PIN;
	//Setting up the port in high level again.
	LPC_GPIO0->FIOSET=1<<WF_RESET_PIN;
	utilsActiveDelay(2000);
	_wiflyUnlock();
}

/*****************************************************************************
 * Function name: _wiflySetCommandMode
 *
 * Descriptions: Sends the string "$$$" through the control UART allowing
 * 				 the device can be configured
 *
 * parameters:
 * 			void
 *
 * Returned value:
 * 			void
 *
 *****************************************************************************/
static void _wiflySetCommandMode(){
	_wiflyLock();
	utilsActiveDelay(1500);
	_wiflySendUart(WF_CMD);
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	if (_wiflyCheckReply(g_wiflyReply,"CMD")){
		g_commandMode=1;
	}else{
		g_commandMode=0;
	}
	if (DEBUGGER){
		if(g_commandMode){
			debugPrintLog("Message: Wifly is now in Command Mode");
		} else{
			debugPrintLog("Message: Could not set up wifly in command mode");
		}
	}
	UARTCleanPort(g_uartWifly);
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	_wiflyUnlock();
}

/*****************************************************************************
 * Function name: _wiflySetNormalMode
 *
 * Descriptions: Takes out the device from Command mode
 *
 * parameters:
 * 			void
 *
 * Returned value:
 * 			void
 *****************************************************************************/
static void _wiflySetNormalMode(){

	if (g_commandMode==1){
		_wiflySendUartLn(WF_EXIT);
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		if (_wiflyCheckReply(g_wiflyReply,"EXIT")){
			g_commandMode=0;
		}
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	UARTCleanPort(g_uartWifly);
}

/*****************************************************************************
 ** Function name:		_wiflyCheckReply
 **
 ** Descriptions:		Function that allows to find a pattern into received
 ** 						g_buffer to check responses of the device
 **
 ** parameters:			char * g_buffer: uart received string
 ** 						char * pattern: substring to looked up into g_buffer
 *
 ** Returned value:		uint8_t 1 if the pattern is found
 ** 								0 if not
 **
 ****************************************************************************/
static uint8_t _wiflyCheckReply (char *g_buffer,char *pattern ){
	char* l_found;
	uint8_t l_response;
	l_found=strstr(g_buffer,pattern);
	if (l_found){
		l_response=1;
	}else{
		l_response=0;
	}
	return l_response;
}

/*****************************************************************************
 * Function name: _wiflySendUartLn();
 *
 * Descriptions:	It provides the same functionality as printfSendUart function
 * 					but also adds a CR+LF code to returning the line
 *
 * parameters:		char* ch: pointer to the values passed to the function
 *
 *
 * Returned value: void
 *
 *****************************************************************************/
void _wiflySendUartLn(const char* chain, ...) {
	_wiflyLock();
	va_list args;
	va_start(args,chain);
	static char l_buff[BUFSIZE];
	vsprintf(l_buff,chain,args);
	strcat(l_buff,"\r\n");
	UARTSend(g_uartWifly,(uint8_t*)l_buff,strlen(l_buff));
	UARTCleanPort(g_uartWifly);
	FREEMEM(l_buff,sizeof(char)*strlen(l_buff));
	_wiflyUnlock();
}


/*****************************************************************************
 * Function name: _wiflySendUart();
 *
 * Descriptions:	Sends the through the initialized Uart the valued passed to
 * 					the function in same format as printf
 *
 * parameters:		uint8_t uart: Uart port to send the data
 * 					char* ch: pointer to the values passed to the function
 *
 *
 * Returned value: void
 *
 *****************************************************************************/
static void _wiflySendUart(const char *ch, ...) {
	_wiflyLock();
	va_list args;
	va_start(args,ch);
	static char l_buff[BUFSIZE];
	vsprintf(l_buff,ch,args);
	va_end(args);
	UARTSend(g_uartWifly,(uint8_t*)l_buff,strlen(l_buff));
	FREEMEM(l_buff,sizeof(char)*strlen(l_buff));
	_wiflyUnlock();
}

/*****************************************************************************
 * Function name: _wiflyGetReply
 *
 * Descriptions:The function reads the UART initialized to work with the wifly
 * 				for the milliseconds  passed as parameter
 *
 * parameters:
 * 			uart: UART Port assigned to the Wifly
 * 			response: pointer to the text received by UART
 * 			timeout: time wait for more data received
 *
 * Returned value:
 *
 *****************************************************************************/
 static uint32_t _wiflyGetReply(char *response, uint32_t timeout){
	_wiflyLock();
	uint32_t l_result;
	l_result=UARTScanf(g_uartWifly,response,timeout);
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflySocketOpen
 **
 ** Descriptions: Function that opens the configured socket by sending the
 ** 			   command "open"
 **
 ** parameters:	 void
 **
 ** Returned value:		uint8_t:
 ** 							1: if connection was successful
 ** 							0 otherwise
 **
 **
 ****************************************************************************/
 uint8_t wiflySocketOpen(){
	 _wiflyLock();
	 uint8_t socketOpen=0;
	 char* openToken="*OPEN*";
	 if (g_commandMode==0){
		 _wiflySetCommandMode();
	 }
	  _wiflySendUartLn(WF_SOCKET_OPEN);
	  wiflyGetServerReply(g_wiflyReply,500);
	  if (utilsCheckValue(g_wiflyReply,openToken)) {
		 g_commandMode=0;
		 socketOpen=1;
	  }
	 UARTCleanPort(g_uartWifly);
	 _wiflyUnlock();
	 return socketOpen;
 }

 /*****************************************************************************
  ** Function name:		wiflySocketClose
  **
  ** Descriptions: Function that forces the disconnection of a
  ** 				socket by sending the comand "close"
  **
  **
  ** parameters:	 void
  **
  ** Returned value:
  ** 	uint8_t :1 If socket was properly closed (server replies *CLOS*)
  ** 			 0	Otherwise
  **
  **
  ****************************************************************************/
 uint8_t wiflySocketClose(){
	 _wiflyLock();
	 uint8_t socketClosed=0;
	 if (g_commandMode==0){
		 _wiflySetCommandMode();
	 }
	 char* closedToken="*CLOS*";
	 socketClosed=wiflySendCommand(WF_SOCKET_CLOSE,closedToken,DEBUGGER);
	 _wiflyUnlock();
	 return socketClosed;
 }

/*****************************************************************************
 ** Function name:		_wiflyIsConnected
 **
 ** Descriptions: Function that checks whether the wifly is associated
 ** 				 to any WLAN or not
 ** parameters:	 void
 **
 **
 ** Returned value:		uint32_t  1: If device is associated
 ** 								  0: Otherwise
 **
 **
 ****************************************************************************/
 uint8_t wiflyIsConnected(){
	 _wiflyLock();
	 uint8_t l_Isconnected=0;
	 if (g_commandMode==0){
		 _wiflySetCommandMode();
	 }

	 l_Isconnected=wiflySendCommand("show n","Assoc=OK",DEBUGGER);
	 g_wiflyConected=l_Isconnected;
	 FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	 if(g_commandMode==1){
		 _wiflySetNormalMode();
	 }
	 _wiflyUnlock();
	 return l_Isconnected;
 }

/*********************************************************PUBLIC FUNCTIONS****************************************************************/


/*****************************************************************************
 * Function name: wiflyInit
 *
 * Descriptions:Initialize the device by setting up the UART port that is going
 * 				to be used or managing the device, also performs a hardware
 * 				reset to the device
 *
 * parameters:
 * 			uart:uart port to be used
 * 			br:baudrate
 *
 * Returned value:
 * 				TRUE or FALSE
 *
 *****************************************************************************/
uint8_t wiflyInit(uint8_t uart,uint32_t br){
	_wiflyLock();
	g_uartWifly=uart;
	uint32_t l_result;
	l_result=UARTInit(g_uartWifly,br); //init the UART again to match the given baudrate as well
	_wiflyReset();
	g_commandMode=0;
	utilsActiveDelay(1000);
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflyConnect
 **
 ** Descriptions:		Makes a connection to a wireless network
 **
 ** parameters:			ssid: ID of the wifi lan to connect
 ** 						passwd: security password of the wireless lan
 ** 						enumeration type of authentication
 **
 ** Returned value:		0 if if succeeded
 **
 ****************************************************************************/

uint8_t wiflyConnect(char *ssid,char *passwd,uint8_t mode) {
_wiflyLock();
	switch (mode)
	{
	case OPEN:
		mode=0;
		break;
	case WEP128:
		mode=1;
		break;
	case WPA1:
		mode=2;
		break;
	case MIXED:
		mode=3;
		break;
	case WPA2:
		mode=4;
		break;
	case Adhoc:
		mode=6;
		break;
	case WPE64:
		mode=7;
		break;
	default:
		mode=0;
		break;
	}

	if(INFO){
		debugPrintLog("***||Trying wireless connection||***");
	}
	utilsActiveDelay(2000);
	if (g_commandMode==0){
		_wiflySetCommandMode();
	}
	wiflySendCommand(utilsMakeChain("set wlan auth %i",mode),WF_OK,DEBUGGER);
	wiflySendCommand(utilsMakeChain("set wlan ssid %s",ssid),WF_OK,DEBUGGER);
	/*Set up depending on the type of authentication selected */
	if(mode==1||mode==7){
		wiflySendCommand(utilsMakeChain("set wlan key %s",passwd),WF_OK,DEBUGGER);
	}else if (mode > 1 && mode <6 ){
			wiflySendCommand(utilsMakeChain("set wlan pass %s",passwd),WF_OK,DEBUGGER);
		}
	/*Ask the the device to join the WLAN and waith some time to complete the connection*/
	wiflySaveConfig();
	wiflyReboot();
	//wiflyJoinWlan();
	utilsActiveDelay(WF_REPLY_DHCP);
	if(g_wiflyConected){
		g_message=utilsMakeChain("***|| Associated to %s Successfully ||***",ssid);
	}else{
		g_message=utilsMakeChain("There was a problem joining %s trying again",ssid);
	}
	if(INFO){
		debugPrintLog(g_message);
	}

	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	if (g_commandMode==1){
		_wiflySetNormalMode();
	}
	_wiflyUnlock();
	return g_wiflyConected;
}

/*****************************************************************************
 ** Function name:		wiflySetBaudrate
 **
 ** Descriptions:		Function to change the Baudrate of the device
 **
 **
 ** parameters:			uint 8: New BaudRate
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetBaudRate(uint32_t br){
	_wiflyLock();
	uint32_t l_result,l_baudrate;
	if (g_commandMode==0){
		_wiflySetCommandMode();
	}
	if(INFO){
		debugPrintLog("***|| Setting new BaudRate to %i ||***\r\n",br);
	}
	if (!g_commandMode){
		_wiflySetCommandMode();
	}
	if (wiflySendCommand(utilsMakeChain("set uart baudrate %i",br),
			WF_OK,DEBUGGER)){
		l_result=1;
		l_baudrate=wiflyGetBaudRate();
		g_message=utilsMakeChain("Baudrate fixed at %i bps.",l_baudrate);
	}else{
		g_message=utilsMakeChain("Error setting up baudrate");
		l_result=0;
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	if (g_commandMode==1){
		_wiflySetNormalMode();
	}
	wiflySaveConfig();
	_wiflyUnlock();
	return l_result;
}


/*****************************************************************************
 ** Function name:		wiflySetTxPw
 **
 ** Descriptions:		Function to change the Transmission power of the
 ** 						Wifly
 **
 **
 ** parameters:			uint 8: New transmission power
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetTxPw(uint8_t newPw){
	_wiflyLock();
	uint32_t l_power=-1;
	/*If no value were passed it takes 0 as value that also means a TX power of 12dB
	 */
	if(newPw==0){
		newPw=12;
	}
	if(INFO){
		debugPrintLog("***|| Setting new Tx Power to %i dB ||***\r\n",newPw);
	}
	if(!g_commandMode){
		_wiflySetCommandMode();
	}
	if(wiflyIsConnected()){
		if (wiflySendCommand(utilsMakeChain("set wlan tx %i",newPw),WF_OK,DEBUGGER)){
			l_power=wiflyGetTxPw();
			g_message=utilsMakeChain("New Tx Power fixed to %i dB",l_power);

		}else{
			g_message=utilsMakeChain("Could not change the power to %i dB",l_power);
			l_power=-1;
		}
	}else{
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		l_power=-1;
	}
	if (INFO){
		debugPrintLog(g_message);
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_power;
}


/*****************************************************************************
 ** Function name:		wiflySetWlanRate
 **
 ** Descriptions:		Function that changes the work rate for connection
 ** 					between the wifly and the wireless access point
 ** 					fulfilling the formula: rate of a v given value is
 ** 					v= Mbps[v-1]+12Mbps
 **
 ** 					Possible values are:
 **
 ** 					3------11Mbps
 ** 					12-----24Mbps (default)
 ** 					15-----54Mbps
 **
 **
 ** parameters:			uint 8: NewRate
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint8_t wiflySetWlanRate(uint8_t newRate){
	_wiflyLock();
	uint32_t l_rate=-1;
	/*If no value were passed it takes 12 as value (24Mbps*/
	if(newRate<3||newRate>15){
		newRate=12;
	}

	if(INFO){
		debugPrintLog("***|| Setting new Rate ||***\r\n",newRate);
	}
	if(!g_commandMode){
		_wiflySetCommandMode();
	}
	if(wiflyIsConnected()){

		if (wiflySendCommand(utilsMakeChain("set wlan rate %i",newRate),WF_OK,DEBUGGER)){
			l_rate=wiflyGetWlanRate();
			g_message=utilsMakeChain("New data rate fixed to %i ",l_rate);

		}else{
			g_message=utilsMakeChain("Could not change the data rate");
			l_rate=24;
		}
	}else{
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		l_rate=24;
	}
	if (INFO){
		debugPrintLog(g_message);
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_rate;
}

/*****************************************************************************
 ** Function name:		WiflySetHotSpot
 **
 ** Descriptions: This function enables the built-in functionality of Software
 ** 				 Access Point mode. In this mode the wifly becomes a DHCP server
 ** 				 an runs a web application that allows you to configure in a
 ** 				 convenient way several parameters such as the SSID and password
 ** 				 in order the wifly can connect to different AP.
 **
 ** 				 The AP' SSID to associated to is WiFly-EZX-02,once you have
 ** 				 associated to it,the DHPC server will leases a 192.168.1.X address.
 **
 ** 				 Then the configuration page should be accessible through a Internet
 ** 				 browser pointing at http://192.168.1.1
 **
 **
 ** parameters:	 void
 **
 **
 ** Returned value:	void
 **
 **
 ****************************************************************************/
void WiflySetHotSpot(){
	_wiflyLock();
	if (!g_commandMode){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("run web_app");
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflySendCommand
 **
 ** Descriptions:		Sends a given command to the wifly devices and capture
 ** 						its response.Also,the function permits to enable the debugger
 ** 						 mode for the command sent.
 **  **
 ** 						Example:
 ** 						Sending the command set ip prococol 2 to wifly should
 ** 						be replied as AOK token, We would use:
 **
 ** 						printfSendCommand ("set ip protocol 2",AOK,1)
 **
 **
 ** parameters:
 ** 					char *  commandToSend: the command to send via UART
 ** 					char *  expectedReply: The token expected to receive
 ** 					uint8_t debugMode:1 to send debug messages
 ** 									  0 if don't
 *
 ** Returned value:		uint8_t 1 if command worked fine
 ** 								0 if there were problems
 **
 ****************************************************************************/
uint32_t wiflySendCommand( char* commandToSend,char* expectedReply,uint8_t debugMode){
	_wiflyLock();
	uint32_t l_result=-1;
	utilsActiveDelay(350);
	if(g_commandMode==0){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn(commandToSend);
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	if (_wiflyCheckReply(g_wiflyReply,expectedReply)){

		if (debugMode){
			debugPrintLog("Comand: %s sent OK",commandToSend);
		}
		l_result=1;
	}
	else{
		if (debugMode){
			debugPrintLog("%s%s --> %s","Command sent: ",commandToSend,"ERROR");
		}
		l_result=0;
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflySendPing
 **
 ** Descriptions:		Function that sends a ping to the host passed as
 ** 						parameter and prints out the result of the ping comand
 ** 						 through the UART port
 **
 ** parameters:			char*: Host to send the ICMP packets
 ** 					uint_8t:number of pings to be sent
 *
 ** Returned value:		uint32_t
 **
 ****************************************************************************/
uint32_t wiflySendPing(char* host, uint8_t times){
	_wiflyLock();
	char l_token[2]=">";
	char l_wiflyLongBuffer[BUFSIZE];
	uint32_t l_result;
	/*if 0 or no value are passed  wifly sends continuous pings, */
	if(times==0){
		times=3;
	}
	g_message=utilsMakeChain("ping %s %i\r\n",host,times);
	if (INFO){
		debugPrintLog("***|| Trying to ping host %s ||***",host);
	}
	if(wiflyIsConnected()){
		if(!g_commandMode){
			_wiflySetCommandMode();
		}
		_wiflySendUartLn("ping %s %i",host,times);
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		_wiflySendUartLn("\r\n");
		sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
		strpbrk(g_buffer,"PING");
		strcpy(l_wiflyLongBuffer,g_buffer);
		g_message=utilsMakeChain("Ping Result %s",l_wiflyLongBuffer);
		l_result=1;
	}
	else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		l_result=0;
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflyGetIp
 **
 ** Descriptions:		Function that shows the devices' ip address through
 ** 						the UART port
 **
 ** parameters:			void
 *
 ** Returned value:		char* pointer to char which contains the ip address
 **
 ****************************************************************************/
void wiflyGetIp(char* ip){
	_wiflyLock();
	if (INFO){
		debugPrintLog("***||Asking for Wifly's IP Address||***");
	}
	if(!g_commandMode){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("get ip a");
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	sprintf(g_buffer,strpbrk(g_wiflyReply,"\n")+1);
	strtok(g_buffer,"\r\n");
	strcpy(ip,g_buffer);
	g_message=utilsMakeChain("Wifly IP address %s",ip);
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	FREEMEM(g_buffer,sizeof(char)*strlen(g_buffer));

	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflyGetSSID
 **
 ** Descriptions:		Function that gets the ssid where the wifly is
 ** 					associated
 **
 ** parameters:			char ssid: Variable to store the ssid names
 *
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflyGetSSID(char* ssid){
	_wiflyLock();
	char* l_token="SSID";
	if (INFO){
		debugPrintLog("***|| Getting AP SSID ||***");
	}
	if(g_wiflyConected){
		if(!g_commandMode){
			_wiflySetCommandMode();
		}
		_wiflySendUartLn("get wlan");
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
		strtok(g_buffer,"\r\n");
		strcpy(ssid,g_buffer);
		g_message=utilsMakeChain("Associated to: %s",ssid);
	}
	else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		ssid="NONE";
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	FREEMEM(g_buffer,sizeof(char)*strlen(g_buffer));
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflyGetPort
 **
 ** Descriptions:		Function that gives the devices' port
 **
 ** parameters:			void
 *
 ** Returned value:		char* pointer to char which contains the port
 **
 ****************************************************************************/
uint16_t wiflyGetPort(){
	_wiflyLock();
	uint16_t l_port;
	char l_token[0x2]=":";
	if (INFO){
		debugPrintLog("***||Asking for Wifly's local Port||***");
	}
	if(!g_commandMode){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("get i");
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	/*format the information received*/
	sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token));
	/*change to a numeric value*/
	l_port=atoi(g_buffer);
	g_message=utilsMakeChain("Wifly port is %i",l_port);
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_port;
}

/*****************************************************************************
 ** Function name:		wiflyGetTxPw
 **
 ** Descriptions:		Prints current transmission power through
 ** 						the UART port
 **
 **
 ** parameters:			void
 *
 ** Returned value:		void
 **
 ***************************************************************************/
uint32_t wiflyGetTxPw(){
	_wiflyLock();
	uint32_t l_pw=-1;
	char* l_token="TxPower";
	if(INFO){
		debugPrintLog("***|| Getting TX POWER ||***");
	}
	/*Check if wifly is connected to any wlan otherwise if not drops off an error g_message*/
	if(g_wiflyConected){
		/*puts the wifly in command mode if it's not yet*/
		if(!g_commandMode){
			_wiflySetCommandMode();
		}
		/*Sends commnad and receive response*/
		_wiflySendUartLn("get wlan");
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		/*format the information received*/
		sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
		strtok(g_buffer,"\r\n");
		/*change to a numeric value*/
		l_pw=atoi(g_buffer);
		/*The 0 value means that wifly's tx power is 12dB (maximum by default)*/
		if (l_pw==0){
			l_pw=12;
		}
		g_message=utilsMakeChain("Wifly Tx Power is %i dB",l_pw);
	}else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));

	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	_wiflyUnlock();
	return l_pw;
}


/*****************************************************************************
 ** Function name:		wiflyGetWlanRate
 **
 ** Descriptions:		Gives the current data rate of connection to the
 ** 					associated wireless access point
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t Current data Rate
 **
 ***************************************************************************/
uint32_t wiflyGetWlanRate(){
	_wiflyLock();
	uint32_t l_rate=12;
	char* l_token=",";
	if(INFO){
		debugPrintLog("***|| Getting WLAN Data Rate ||***");
	}
	/*Check if wifly is connected to any wlan otherwise if not drops off an error g_message*/
	if(g_wiflyConected){
		/*puts the wifly in command mode if it's not yet*/
		if(!g_commandMode){
			_wiflySetCommandMode();
		}
		/*Sends commnad and receive response*/
		_wiflySendUartLn("get wlan");
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		/*format the information received*/
		sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
		strtok(g_buffer,"\r\n");
		/*change to a numeric value*/
		l_rate=atoi(g_buffer);

		g_message=utilsMakeChain("Current Wifly data Rate is %i Mb",l_rate);
	}else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	return l_rate;
}

/*****************************************************************************
 ** Function name:		wiflyGetRSSI
 **
 ** Descriptions:		Prints current rssi value of wifly through
 ** 						the UART port
 **
 **
 ** parameters:			void
 *
 ** Returned value:		int16_t the obtained RSSI value
 **
 ****************************************************************************/
int16_t wiflyGetRSSI(){
	_wiflyLock();
	char *l_token="(";
	int32_t l_rssi;
	if(INFO){
		debugPrintLog("***|| Getting RSSI VALUE ||***");
	}
	if(g_commandMode==0){
		_wiflySetCommandMode();
	}
	_wiflyLock();
	_wiflySendUartLn("show rssi");
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	_wiflyUnlock();
	sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token));
	strtok(g_buffer,")");
	l_rssi=atoi(g_buffer);
	g_message=utilsMakeChain("RSSI value:%i",l_rssi);
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	_wiflyUnlock();
	return l_rssi;
}

/*****************************************************************************
 ** Function name:		GetUptime
 **
 ** Descriptions:		Prints out through the UART port  the amount of time
 ** 						in which the device have remained powered on
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t Number of seconds that the wifly has been
 ** 								 powered on
 ** 								-1: in case of error
 **
 ****************************************************************************/
uint32_t wiflyGetUptime(){
	_wiflyLock();
	char *l_token="UpTime";
	char l_time[5];
	uint32_t l_uptime;
	if(INFO){
		debugPrintLog("***|| Getting UPTIME VALUE ||***");
	}
	if (g_commandMode==0){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("show time");
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	sprintf(l_time,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
	strtok(l_time,"s");
	l_uptime=atoi(l_time);
	g_message=utilsMakeChain("Uptime is:%i s",l_uptime);
	if(INFO){
		debugPrintLog(g_message);
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_uptime;
}

/*****************************************************************************
 ** Function name:		wiflyGetChannel
 **
 ** Descriptions:		Prints out through the UART port the wlan channel of
 ** 						the access point that it's currently associated
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t
 **
 ****************************************************************************/
uint32_t wiflyGetChannel(){
	_wiflyLock();
	char *l_token="Chan";
	uint32_t l_ch;
	if (INFO){
		debugPrintLog("***|| Getting WIFI Channel ||***");
	}
	if(wiflyIsConnected()){

		if(!g_commandMode){
			_wiflySetCommandMode();
		}
		utilsActiveDelay(1000);
		_wiflySendUartLn("show n");
		_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
		sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
		strtok(g_buffer,"\r\n");
		l_ch=atoi(g_buffer);
		g_message=utilsMakeChain("Wifly's wlan Channel %i",l_ch);
	}
	else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		l_ch=-1;
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	if(g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	FREEMEM(g_buffer,sizeof(char)*strlen(g_buffer));
	_wiflyUnlock();
	return l_ch;
}

/*****************************************************************************
 ** Function name:		wiflyGetBaudRate
 **
 ** Descriptions:		Function to ask the device for its current Baudrate
 ** 						value and print it through UART Debug Port
 **
 **
 ** parameters:			uint 8: void
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint32_t wiflyGetBaudRate(){
	_wiflyLock();
	char *l_token="Baudrate";
	uint32_t l_baudrate;
	if (INFO){
		debugPrintLog("***|| Getting Baudrate value ||***");
	}
	if (g_commandMode==0){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("get uart");
	_wiflyGetReply(g_wiflyReply,WF_REPLY_TIMEOUT);
	sprintf(g_buffer,"%s",strstr(g_wiflyReply,l_token)+strlen(l_token)+1);
	strtok(g_buffer,"\r\n");
	l_baudrate=atoi(g_buffer);
	g_message=utilsMakeChain("Current BaudRate is %i",l_baudrate);
	if(INFO){
		debugPrintLog(g_message);
	}
	if (g_commandMode==1){
		_wiflySetNormalMode();
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	FREEMEM(g_buffer,sizeof(char)*strlen(g_buffer));
	_wiflyUnlock();
	return l_baudrate;
}

/**************************************************************************
 ** Function name:		wiflyDisconnect
 **
 ** Descriptions:		Function to disconnect the device from the WLAN
 **
 **
 ** parameters:			void
 *
 ** Returned value:		uint32_t 1: If command was successful
 ** 								 0: If there was any error
 **
 ****************************************************************************/
uint32_t wiflyDisconnect(){
	_wiflyLock();
	uint32_t l_result;
	if(!g_commandMode){
		_wiflySetCommandMode();
	}
	if (wiflyIsConnected()){
		wiflySendCommand(utilsMakeChain(WF_DISCONNECT_WLAN),"DeAuth",DEBUGGER);
		l_result=1;
	}else {
		g_message=utilsMakeChain(WF_NOT_CONECTED_MSG);
		l_result=0;
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	FREEMEM(g_message,sizeof(char)*strlen(g_message));
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflyReboot
 **
 ** Descriptions:		Performs a reset on the device
 **
 **
 ** parameters:			void
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflyReboot(){
	_wiflyLock();
	if(g_commandMode==0){
		_wiflySetCommandMode();
	}
	wiflySendCommand("reboot","*Reboot*",DEBUGGER);
	g_commandMode=0;
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflySaveConfig
 **
 ** Descriptions:		Function to store the configuration in the device's
 ** 						non-volatile memory
 **
 ****
 ** parameters:			void
 ** Returned value:		void
 **
 ****************************************************************************/
void wiflySaveConfig(){
	_wiflyLock();
	if (!g_commandMode){
		_wiflySetCommandMode();
	}
	if(INFO){
		debugPrintLog("***|| Storing configuration ||***\r\n");
	}
	if(wiflySendCommand("save","Storing",DEBUGGER)){
		g_message="Configuration saved!";
	}else{
		g_message="Error: Configuration not Stored";
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflyjoinWlan
 **
 ** Descriptions: Function that forces to connect the device to a configured
 ** 				 wlan. If device was already connected the function forces to
 ** 				 re-connect
 **
 ** parameters:	 void
 ****
 ** Returned value:		uint32_t  1: If device is associated
 ** 								  0: Otherwise
 **
 **
 ****************************************************************************/
void wiflyJoinWlan(){
	_wiflyLock();
	if (!g_commandMode){
		_wiflySetCommandMode();
	}
	_wiflySendUartLn("join");
	utilsActiveDelay(2000);
	_wiflyGetReply(g_wiflyReply,WF_REPLY_DHCP);
	if (_wiflyCheckReply(g_wiflyReply,"IF=UP")){
		g_wiflyConected=1;
	}
	utilsActiveDelay(WF_REPLY_DHCP);
	_wiflySetNormalMode();
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		 wiflyRemoteConnectionSetup
 **
 ** Descriptions: Function that configure the wifly to connect to a generic
 ** 				 remote server
 **
 ** parameters:			uint8_t protocol: type of socket.
 ** 									HTTP,TCP client/server UDP etc.
 ** 					uint8_t remoteIp: The remote host ip address
 ** 						 				(0	if dns name is active)
 **
 ** 					uint8_t portRemote: remote socket port
 **
 ** 					char* host: In case of conection must be done
 ** 								by the server FQDN address
 **
 **
 ** Returned value:		uint8_t  1: If the configuration is successful
 ** 							 0: Otherwise
 **
 **
 ****************************************************************************/
uint8_t wiflyRemoteConnectionSetup(uint8_t protocol,char* remoteIp,uint16_t portRemote,
		uint16_t portLocal,char* host)
{
	_wiflyLock();
	uint8_t l_result=0;
	uint32_t l_wiflylocalPort=0;
	uint32_t comSize=1420;
	uint32_t comTime=100 ;
	uint32_t idleTimeout=8;

	if(INFO){
		debugPrintLog("***||Trying remote server connection||***");
	}
	if (portLocal!=0){
		l_wiflylocalPort=portLocal;
	}
	/*Here is the configuration of protocol HTTP,TCP,UDP,etc. and the port where server will
	 * be listened*/
	l_result =wiflySendCommand(utilsMakeChain("set ip protocol %i",protocol),WF_OK,DEBUGGER);
	l_result &=wiflySendCommand(utilsMakeChain("set ip remote %i",portRemote),WF_OK,DEBUGGER);
	l_result &=wiflySendCommand(utilsMakeChain("set com remote 0"),WF_OK,DEBUGGER);
	/* Other communication setup*/
	l_result =wiflySendCommand(utilsMakeChain("set com size %i",comSize),WF_OK,DEBUGGER);// Set the maximum flush size
	l_result &=wiflySendCommand(utilsMakeChain("set com time %i",comTime),WF_OK,DEBUGGER);// Set the maximum flush time
	l_result &=wiflySendCommand(utilsMakeChain("set com idle %i",idleTimeout),WF_OK,DEBUGGER);// Set the idle timeout

	/*In case of connect to a server by its FQDN address, the wifly has to be setup with
	 * command "set ip host 0" and then the dns name must be configured on the wifly
	 * The other configuration in common to any other servers setup.
	 * No needed to change the wifly's listen port                                      */
	if(remoteIp==0){
		l_result &=wiflySendCommand(utilsMakeChain("set ip host %s",host),WF_OK,DEBUGGER);
		l_result  &=wiflySendCommand(utilsMakeChain("set dns name %s",host),WF_OK,DEBUGGER);
		l_result &=wiflySendCommand("set option format 1",WF_OK,DEBUGGER);
	}
	/*
	 * Otherwise the wifly will be setup with the remote server IP address and also
	 * it would be necessary to change the port socket where wifly is listening, in case of
	 * TCP or UDP configurations.
	 * */
	else{
		l_result &=wiflySendCommand(utilsMakeChain("set ip host %s",remoteIp),WF_OK,DEBUGGER);
		l_result &=wiflySendCommand(utilsMakeChain("set ip local %i",l_wiflylocalPort),WF_OK,DEBUGGER);
	}
	if (l_result){
		if(remoteIp==0){
			g_message=utilsMakeChain("***|| Connection to %s:%i finished OK ||***",host,portRemote);
		}else{
			g_message=utilsMakeChain("***|| Connection to %s:%i finished OK ||***",remoteIp,portRemote);
		}
		g_serverConfigured=1;
		wiflySaveConfig();
	}
	else{
		g_message=utilsMakeChain("***|| ERROR Setup failed ||***");
	}
	if(INFO){
		debugPrintLog(g_message);
	}
	FREEMEM(g_wiflyReply,sizeof(char)*strlen(g_wiflyReply));
	_wiflyUnlock();
	return l_result;
}

/*****************************************************************************
 ** Function name:		wiflyCommandtoServer
 **
 ** Descriptions: Prepares the command to be sent to the remote server
 **
 ** parameters: char* command: Command to make up
 **
 **
 ** Returned value: void
 **
 ****************************************************************************/
void wiflyCommandtoServer(char* command){
	_wiflyLock();
	if (g_commandMode==0){
		_wiflySetCommandMode();
	}
	if(g_wiflyConected){
		_wiflySendUartLn("set com remote %s",command);
		_wiflySendUartLn(WF_SOCKET_OPEN);
		if (wiflySendCommand("\r\n","*CLOS*",DEBUGGER)){
			g_commandMode=0;
		}else{
			debugPrintLog(WF_SERVER_ERROR);
		}
		UARTCleanPort(g_uartWifly);
	}
	else{
		debugPrintLog(WF_NOT_CONECTED_MSG);
	}
	_wiflyUnlock();
}

/*****************************************************************************
 ** Function name:		wiflyGetServerReply
 **
 ** Descriptions: Function to get the response from the remote server
 ** 				by listening to the UART port assigned to communicate
 ** 				to the device for a while
 **
 ** parameters: char* reply: pointer where the response must be stored
 ** 			uint32_t timewait: timeout for listening the response
 **
 ** Returned value: void
 **
 ****************************************************************************/
void wiflyGetServerReply(char* reply,uint32_t timeWait){
	_wiflyLock();
	if(timeWait==0){
		timeWait=TCP_LISTEN_TIMEOUT;
	}
	_wiflyGetReply(reply,timeWait);
	if(utilsCheckValue(g_wiflyReply,"ERR:Connected!")){
		g_commandMode=1;
	}else{
		g_commandMode=0;
	}
	_wiflyUnlock();
}




