
/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "basic_io.h"

/*UART and Mutex Includes*/
#include "type.h"
#include "uart.h"
#include "semphr.h"
#include "uprintf.h"
#include "debug.h"
#include "LPC17xx.h"
#include "configureMe.h"
#include "motor.h"


#define COMMANDBUFFER	0xFF

/* The tasks to be created.  Two instances are created of the sender task while
 only a single instance is created of the receiver task. */
static void vSystemInitTask (void *pvParameters);
static void vGetCommandTask(void *pvParameters);
static void vExecCommandTask(void *pvParameters);
static void vGetSignalTask (void *pvParameters);



static xTaskHandle xSystemInitTaskHandler;
static xTaskHandle xGetCommandTaskHandler;
static xTaskHandle xExecCommandTaskHandler;
static xTaskHandle xGetSignalTaskHandler;




xQueueHandle xQueue;

volatile int16_t g_rssiThreshold=-99;
int16_t g_currentRssi;
volatile uint32_t g_currentDistance;


/*-----------------------------------------------------------*/
 int main(void) {

	char* response[0xFF];



	xQueue = xQueueCreate(4, sizeof(long));
	if (xQueue != NULL) {
		/*Task to obtain the data sent by the remote controller*/
		xTaskCreate(vSystemInitTask, "Init", 240, NULL, 3,&xSystemInitTaskHandler);
		/*Task to obtain the data sent by the remote controller*/
		xTaskCreate(vGetCommandTask, "Get", 240, NULL, 2,&xGetCommandTaskHandler);
		/*Task to send the data to the motorControl for execute the commandReceiver task creation*/
		xTaskCreate(vExecCommandTask, "Exec", 240, NULL, 3,&xExecCommandTaskHandler);
		/*Task to periodically ask current RSSI level*/
		xTaskCreate(vGetSignalTask, "Signal", 240, NULL, 1,&xGetSignalTaskHandler);


		/*Start of scheduler process*/
		vTaskStartScheduler();

		}
		/* If all is well we will never reach here as the scheduler will now be
	 running.  If we do reach here then it is likely that there was insufficient
	 heap available for the idle task to be created. */
		for (;;)
			;
		return 0;


}
/*-----------------------------------------------------------*/

void vSystemInitTask (void *pvParameters){
	vTaskSuspend(xExecCommandTaskHandler);
	vTaskSuspend(xGetCommandTaskHandler);
	vTaskSuspend(xGetSignalTaskHandler);
	pingInit(ADC0);
	motorInit(MOTOR_UART);
	wiflyInit(WIFLY_UART, WIFLY_BAUDRATE);
	debugInit(DEBUG_UART, UART_BAUDRATE);
	if (DEBUGGER){
	motorTests();
	}
	if (!wiflyIsConnected()){
		wiflyConnect(WIFI_SSID,WIFI_PWD,WPA2);
	}
	/*TCP remote communication setup*/
	vilmaControlRemoteInit(REMOTE_IP,REMOTE_PORT,WIFLY_LOCAL_PORT,0);
	vTaskResume(xGetCommandTaskHandler);
	/*Finishing up this task*/
	vTaskDelete(NULL);

}


/*Task that obtains the command sent by the remote controller */

void vGetCommandTask(void *pvParameters) {
	char* commandToSend;
	uint16_t commandLength=0;
	portBASE_TYPE xStatus;
	uint32_t waitToSpace = 500;
	const portTickType xTickToWait = waitToSpace / portTICK_RATE_MS;

	for (;;) {
		commandToSend=(char*)controlServerGetSocketData();

		if(commandToSend!=NULL){
			xStatus = xQueueSendToBack(xQueue, &commandToSend, waitToSpace);
			if (xStatus != pdPASS) {
				debugPrintLog("Queue is full!");
			} else {
				debugPrintLog("Command inserted to the queue:%s\r\n",commandToSend);
				vTaskResume(xExecCommandTaskHandler);
				vTaskResume(xGetSignalTaskHandler);
			}
			taskYIELD();
		}
	}
}

/*TASK that executes the received command*/
void vExecCommandTask(void *pvParameters) {
	uint32_t receivedValue;
	char* message;
	portBASE_TYPE xStatus;
	uint32_t delayReceive=500;
	const portTickType xTicksToWait = delayReceive / portTICK_RATE_MS;
	for (;;) {
		if (uxQueueMessagesWaiting(xQueue) != 0) {
			debugPrintLog("There are pending commands in the queue\r\n");
		}
		xStatus = xQueueReceive( xQueue, &message, xTicksToWait );
		if (xStatus == pdPASS){
			if (utilsCheckValue(message,"RSSI")){
				vilmaControlExecCommand(message);
				_vilmaControlSendStatus();

			}else{
				if (g_currentRssi<=g_rssiThreshold)	{
					vilmaControlSendSocketData("msg=WARNING:SIGNAL_RATE_TOO_LOW!!!");
					motorUndoMovement();
				}else if(utilsCheckValue(message,"F") && (g_currentDistance<=BUMP_DISTANCE)){
					vilmaControlSendSocketData(utilsMakeChain("msg=WARNING:VILMA_ABOUT_TO_CRASH!!!"));
					motorStop();
					_vilmaControlSendStatus();
				}else{
					vilmaControlExecCommand(message);
					_vilmaControlSendStatus();
				 	vTaskSuspend(xExecCommandTaskHandler);
				}
			}
			vTaskSuspend(xExecCommandTaskHandler);
		}

	}
}
/*Periodic task to get the current RSSI*/
void vGetSignalTask (void *pvParameters){
	portBASE_TYPE xLastWakeTime;
	xLastWakeTime=xTaskGetTickCount();
	uint16_t repeatingTime=40000;

	for(;;){
		vTaskDelayUntil(&xLastWakeTime,(repeatingTime/portTICK_RATE_MS));
		g_currentRssi=wiflyGetRSSI();
	}
}







/*-----------------------------------------------------------*/

 void vApplicationMallocFailedHook(void) {
	 /* This function will only be called if an API call to create a task, queue
	 or semaphore fails because there is too little heap RAM remaining. */
	 printf("Run out of memory!");

	 for (;;){
		 ;
	 }
 }
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook(xTaskHandle *pxTask, signed char *pcTaskName) {
	/* This function will only be called if a task overflows its stack.  Note
	 that stack overflow checking does slow down the context switch
	 implementation. */
	for (;;){
		;
	}
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook(void) {

	}
/*-----------------------------------------------------------*/

void vApplicationTickHook(void) {
	/* This example does not use the tick hook to perform any processing. */
}

